/**********************************************************************/
/*                                                                    */
/* Dispatches and collects mail using RexxMail.CMD and MailPOP3.CMD.  */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program forms part of the RexxMail package, and may not be    */
/* distributed separately.                                            */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use RexxComm.CMD to reformat the comments in this program.         */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* The latest versions of RexxMail and RexxComm can be found at       */
/*               www.degeus.com/rexx                                  */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program is released under the terms of the GNU license, see   */
/*               www.gnu.org/copyleft/gpl.html                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* (c) 2000-2001 Marcus C. de Geus                                    */
/*               marcus@degeus.com                                    */
/*               www.degeus.com                                       */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* History:                                                           */
/*                                                                    */
/* 2001-04-03: Corrected "chaining error" bug when using CMD.EXE.     */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use it if you like it. Don't if you don't. No legalese.            */
/*                                                                    */
/**********************************************************************/

signal on halt  /* handle halt condition */

/**********************************************************************/
/* Call RexxMail.CMD to dispatch outgoing messages                    */
/**********************************************************************/

parse arg MailFiles  /* get any argument */

'call rexxmail.cmd /dispatch '||MailFiles  /* use RexxMail to dispatch messages or generate a reply */

if (RC >< 0) then  /* if the return code is not 0 , i.e. an error occurred, or the "Dispatch" object was used to create a reply message */
do
 call Halt  /* quit */
end

/**********************************************************************/
/* Collect incoming messages                                          */
/**********************************************************************/

/**********************************************************************/
/* Start by looking for the current user in the OS/2 environment      */
/**********************************************************************/

CurrentUser = value('USER',,'OS2ENVIRONMENT')  /* get the current user's name */

if (CurrentUser = '') then  /* if we find no user name in the environment */
do
 call beep 333,333  /* signal */
 say 'No user name found in OS/2 environment'  /* report */
 call Halt  /* and abort */
end

/**********************************************************************/
/* See if the locations file can be found and used                    */
/**********************************************************************/

parse source . . ProgSpec  /* get the program name */
ProgDir = filespec('D',ProgSpec)||filespec('P',ProgSpec)  /* get the program directory */
LocFile = ProgDir||'location.txt'  /* the name of the locations file */

if (stream(LocFile,'C','QUERY EXISTS') = '') then  /* if we cannot find the location file */
do
 call beep 333,333  /* signal */
 say 'Could not find "'||LocFile||'"'  /* report */
 call Halt  /* and abort */
end

if (stream(LocFile,'C','OPEN READ') >< 'READY:') then  /* if we cannot open the location file */
do
 call beep 333,333  /* signal */
 say 'Could not open "'||LocFile||'"'  /* report */
 call Halt  /* and abort */
end

/**********************************************************************/
/* Get the current user's main file location from location.txt        */
/**********************************************************************/

CurrentLocation = ''  /* start with nothing */
do while (lines(LocFile))  /* as long as we have lines left */
 NextLine = linein(LocFile)  /* get the next one */
 NextLine = translate(NextLine,d2c(32),d2c(9))  /* convert TABs to spaces */
 if (NextLine >< '') then  /* if the line is not empty */
 do
  if (substr(NextLine,1,1) >< '#') then  /* if it is not a comment */
  do
   parse var NextLine Location ' = ' User ';'  /* get the bits we want */
   User = translate(strip(User))  /* get rid of excess spaces and make it upper case */
   if User = translate(CurrentUser) then  /* if it is the one we want */
   do
    CurrentLocation = Location  /* this is the location we want */
   end
  end
 end
end

call stream LocFile,'C','CLOSE'  /* close the locations file */

if (CurrentLocation = '') then  /* if we have no current location */
do
 call beep 333,333  /* signal */
 say 'Could not find location for "'||CurrentUser||'"'  /* report */
 call Halt  /* quit */
end

/**********************************************************************/
/* See if the POP3 response lines file can be found                   */
/**********************************************************************/

RespFile = CurrentLocation||'\configuration\pop3resp.txt'  /* the name of the response file to use */

if (stream(RespFile,'C','QUERY EXISTS') = '') then  /* if we cannot find the response file */
do
 say 'Could not find "'||RespFile||'"'  /* report */
 call Halt  /* and abort without a signal, so this can be used to escape POP3 processing */
end

/**********************************************************************/
/* See if the POP3 response lines file can be opened                  */
/**********************************************************************/

if (stream(RespFile,'C','OPEN READ') >< 'READY:') then  /* if we cannot open the response file */
do
 call beep 333,333  /* signal */
 say 'Could not open "'||RespFile||'"'  /* report */
 call Halt  /* and abort */
end

/**********************************************************************/
/* Determine the values of the entries in the POP3 response file      */
/**********************************************************************/

RespEntries = 'SERVERNAME',  /* the possible POP3 response entries are the server name, */
              'USERNAME',  /* the user name, */
              'PASSWORD',  /* the password, */
              'RECEIVEDIR',  /* the receive dir, */
              'INTERACTIVE'  /* and the "Interactive" parameter */

drop POP3.  /* start with nothing */

do while (lines(RespFile))  /* go on until we run out of lines */
 Line = linein(RespFile)  /* read the next line */
 Line = translate(Line,d2c(32),d2c(9))  /* convert TABs to spaces */
 Line = strip(Line,'B',' ')  /* get rid of excess spaces */
 if (Line >< '') then  /* if the line is not empty */
 do
  if (substr(Line,1,1) >< '#') then  /* if it is not a comment */
  do
   Line = translate(Line,d2c(32),d2c(9))  /* turn TABs into spaces */
   parse var Line RespEntry '=' RespValue  /* get the parts we need */
   RespEntry = translate(strip(RespEntry))  /* get rid of superfluous spaces and make it upper case */
   if (wordpos(RespEntry,RespEntries) > 0) then  /* if it is one of the range of possible entries */
   do
    POP3.RespEntry = strip(RespValue)  /* use the value without superfluous spaces */
   end
   else  /* if it is not a valid entry */
   do
    call beep 333,333  /* signal */
    say 'Invalid entry in POP3 response file "'||RespEntry||'"'  /* report */
    call Halt  /* quit */
   end
  end
 end
end

call stream RespFile,'C','CLOSE'  /* close the POP3 response file */

/**********************************************************************/
/* Give up if any of the parameters is missing                        */
/**********************************************************************/

if (words(POP3.ServerName,  /* if the number of words in the string containing the server name, */
          POP3.UserName,  /* the user name, */
          POP3.Password,  /* the password, */
          POP3.ReceiveDir) < 4) then  /* and the receive directory is less than 4 */
do
 say 'One or more entries are missing from "'||RespFile||'"'  /* report - no beep, to provide an escape for out-of-the-box operation */
 call Halt  /* and quit */
end

/**********************************************************************/
/* Pass on the Interactive flag if necessary                          */
/**********************************************************************/

if (translate(POP3.Interactive = 'YES')) then  /* if the Interactive flag was set */
do
 POP3.Interactive = 'I'  /* use "I" */
end
else  /* if the flag was not set */
do
 POP3.Interactive = ''  /* use nothing */
end

/**********************************************************************/
/* Call the MailPOP3.CMD program using the values we have found       */
/**********************************************************************/

'call mailpop3.cmd '||POP3.ServerName,  /* call MailPOP3.CMD with the server name, */
                      POP3.UserName,  /* the user name, */
                      POP3.Password,  /* the password, */
                      POP3.ReceiveDir,  /* the dir in which to drop incoming mail mesages, */
                      POP3.Interactive  /* and possibly the Interactive parameter, to collect any waiting mail */

/**********************************************************************/
/* Use RexxMail.CMD to process any POP3 mail messages we collected    */
/**********************************************************************/

call sysfiletree ReceiveDir||'\POP3???.MES','Files.','FO'  /* look for message files in the receive dir */
do Index = 1 to Files.0  /* take each of the files */
 'call rexxmail.cmd /register '||Files.Index  /* and use rexxmail to register it (i.e. label it and drop it into the RexxMail "In" folder) */
end

/**********************************************************************/
/* The end of the road                                                */
/**********************************************************************/

Halt:  /* handles halt condition */

exit  /* that's all, folks! */
