/**********************************************************************/
/*                                                                    */
/* Gets waiting mail messages from POP3 server                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Release history:                                                   */
/*                                                                    */
/* 2001-02-22 : Original program release.                             */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use RexxComm.CMD to reformat the comments in this program.         */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* The latest version of this program and RexxComm can be found at    */
/*               www.degeus.com/rexx                                  */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program is released under the terms of the GNU license, see   */
/*               www.gnu.org/copyleft/gpl.html                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* (c) 2000-2001 Marcus C. de Geus                                    */
/*               marcus@degeus.com                                    */
/*               www.degeus.com                                       */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* History:                                                           */
/*                                                                    */
/* 2001-04-03: Corrected "chaining error" bug when using CMD.EXE.     */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use it if you like it. Don't if you don't. No legalese.            */
/*                                                                    */
/**********************************************************************/

signal on Halt  /* handle breaks */

parse source . . ProgSpec  /* get ProgSpec */
ProgName = filespec('N',ProgSpec)  /* just the program name */

Messages = 0  /* start with no result */

parse arg ServerName UserName Password RecvDir Interactive  /* get the arguments */

if (RecvDir = '') then  /* if the last one is missing */
do
 say 'Usage : '||ProgName||' ServerName UserName Password ReceiveDir [I(nteractive)]'  /* report */
 say ''  /* empty line */
 say 'ServerName    = name of the POP3 server'  /* report */
 say 'UserName      = your user name at the POP3 server'  /* report */
 say 'Password      = your password to access the POP3 user account'  /* report */
 say 'ReceiveDir    = directory where message files should be stored'  /* report */
 say 'I(nteractive) = operate in interactive mode:'  /* report */
 say '                list waiting messages and prompt for action to take'  /* report */
 say ''  /* empty line */
 say 'Note 1: Non-interactive mode will list all waiting messages,'  /* report */
 say '        then collect each one, deleting it from the server.'  /* report */
 say ''  /* empty line */
 say 'Note 2: The program exit code indicates the number of messages found.'  /* report */
 exit  /* and quit */
end

if (\LoadRexxUtils()) then  /* if we cannot use or load the Rexx utility functions */
do
 call Halt  /* exit */
end

if (\LoadRexxSocks()) then  /* if we cannot use or load the Rexx socket functions */
do
 call Halt  /* exit */
end

say 'Looking for mail at '||ServerName  /* report */

Interactive = (translate(substr(Interactive,1,1)) = 'I')  /* if we have an "I..." switch, we're going interactive */

ServName = translate(ServerName,'_','.')  /* replace dots with underscores */

if (\sockgethostbyname(ServerName,'Host.!')) then  /* if we cannot get the server name */
do
 say 'Cannot resolve server name '||ServerName  /* report */
 call Halt  /* and quit */
end

Server = Host.!Addr  /* the bit we want */

Sock = socksocket('AF_INET','SOCK_STREAM','IPPROTO_TCP')  /* get the socket */
if (Sock = -1) then  /* if we get an error */
do
 say 'Cannot open socket'  /* report */
 call Halt  /* and quit */
end

Server.!family = 'AF_INET'  /* set the server family */
Server.!port = 110  /* the port to use */
Server.!addr = Server  /* the address to use */

if (sockconnect(Sock,'Server.!') = -1) then  /* if we cannot connect to the server */
do
 say 'Cannot connect to POP server'  /* report */
 call socksoclose(Sock)  /* close the socket */
 call Halt  /* and quit */
end

TRC = GetResponse(Sock)  /* get a response from the server */
TRC = sendmessage(Sock,'USER '||UserName)  /* send the user name to the server */
TRC = GetResponse(Sock)  /* get a response from the server */

parse var Line.1 Status .  /* get the status text */
if Status >< '+OK' then  /* if the status is not "+OK" */
do
 say 'User '||UserName||' unknown on '||ServerName  /* report */
 TRC = sendmessage(Sock,'QUIT')  /* quit from the server */
 TRC = socksoclose(Sock)  /* close the port */
 call Halt  /* and quit */
end

TRC = sendmessage(Sock,'PASS '||Password)  /* send the password to the server */
TRC = GetResponse(Sock)  /* get a response */

parse var Line.1 Status .  /* get the status text */
if Status >< '+OK' then  /* if the status is not "+OK" */
do
 say 'Incorrect password'  /* report */
 TRC = SendMessage(Sock,'QUIT')  /* quit from the server */
 TRC = socksoclose(Sock)  /* close the port */
 call Halt  /* and quit */
end

TRC = SendMessage(Sock,'LIST')  /* ask the server for a list of waiting messages */
TRC = GetResponse(Sock)  /* REXX port stuff */

Messages = 0  /* start with nothing */
msginfo = GetResponseLine(Sock)  /* get a line */
do while MsgInfo >< '.'  /* as long as we have something other than a full stop */
 parse var Msginfo Messages Size  /* get the message number and size */
 say 'Message '||Messages||' has '||Size||' bytes'  /* report */
 msginfo = GetResponseLine(Sock)  /* get the next line */
end

if (Messages = 0) then  /* if there are no messages */

do
 say 'There are no messages'  /* report */
end

else  /* if there are messages to collect */

do

 if (Messages > 1) then  /* if we have more than 1 message */
 do
  Plural = 's'  /* we need a plural "s" */
 end
 else  /* if we have exactly one message */
 do
  Plural = ''  /* we do not need a plural "s" */
 end

 if (Interactive) then  /* if we are going interactive */
 do
  say 'You have '||Messages||' waiting message'||Plural  /* report */
 end
 else  /* if we are not interactive */
 do
  say 'Collecting '||Messages||' message'||Plural  /* report */
 end

 do Index = 1 to Messages  /* take each message */

  if (Interactive) then  /* if we're doing things interactively */
  do
   call charout StdOut,'Retrieve message no. '||Index||'? [Y/N] Y'||d2c(8)  /* user prompt */
   InKey = sysgetkey('ECHO')  /* get a response */
   say ''  /* skip to the next line */
   if (pos(InKey,'Yy+'||d2c(13)) > 0) then  /* if it is Y(es) or [Enter] */
   do
    call RetrieveMess  /* retrieve the message */
   end
   call charout StdOut,'Delete message '||Index||' from server? [Y/N] N'||d2c(8)  /* user prompt */
   InKey = sysgetkey('ECHO')  /* get a response */
   say ''  /* skip to the next line */
   if (pos(InKey,'Yy+') > 0) then  /* if it is Y(es) */
   do
    call DeleteMess  /* delete the message */
   end
  end

  else  /* if we're not interactive */

  do
   if (RetrieveMess()) then  /* if the message can be retrieved */
   do
    call DeleteMess  /* delete the message */
   end
  end

 end

end

TRC = SendMessage(Sock,'QUIT')  /* quit from the server */
TRC = GetResponse(Sock)  /* get a response */
TRC = SockSoclose(Sock)  /* close the port */

call Halt  /* that's all, folks! */

/**********************************************************************/
RetrieveMess:  /* retrieve a message from the server */
/**********************************************************************/

say 'Retrieving message '||Index  /* report */
TRC = SendMessage(Sock,'RETR '||Index)  /* ask to retrieve the message from the server */
TRC = GetResponse(Sock)  /* get a response */
parse var Line.1 Status .  /* look for the status text */
if (Status >< '+OK') then  /* if the command was not accepted */
do
 say 'Could not retrieve message '||Index||' from server'  /* report */
 Success = 0  /* no success */
end
else  /* if the command was accepted */
do
 MailFile = SysTempFileName(RecvDir||'\POP3????.MES')  /* get a unique file name in the received mail folder */
 if (stream(MailFile,'C','OPEN WRITE') = 'READY:') then  /* if we can open the file for writing */
 do
  OneLine = GetResponseLine(Sock)  /* get a line */
  do while (OneLine >< '.')  /* as long as it is not a full stop */
   call lineout MailFile,OneLine  /* write the line to the file */
   oneline = GetResponseLine(Sock)  /* get the next line */
  end
  call stream MailFile,'C','CLOSE'  /* close the file */
  say 'Retrieved message '||Index  /* report */
  Success = 1  /* all is well */
 end
 else  /* if we cannot open the file */
 do
  say 'Cannot open file : '||MailFile  /* report */
  Success = 0  /* no success */
 end
end

return Success  /* end of RetrieveMess */

/**********************************************************************/
DeleteMess:  /* delete a message from the server */
/**********************************************************************/

TRC = SendMessage(sock,'DELE '||Index)  /* ask the server to delete the message */
TRC = GetResponse(sock)  /* get the response */
parse var Line.1 Status .  /* look for the status text */
if (Status >< '+OK') then  /* if the command was not accepted */
do
 say 'Could not delete message '||Index||' from server'  /* report */
 Success = 0  /* no success */
end
else  /* if the command was accepted */
do
 say 'Deleted message '||Index  /* report */
 Success = 1  /* all is well */
end

return Success  /* end of DeleteMess */

/**********************************************************************/
GetResponse:     procedure expose !. Line.  /* get a response from the server */
/**********************************************************************/

parse arg Sock  /* get the argument */
MoreIDs = '100 215 220 221 222 223 230 231'  /* codes indicating more */
LineCount = 1  /* start counting at 1 */
Line.LineCount = GetResponseLine(Sock)  /* get a line */
parse var Line.1 RID .  /* get the return code from the line */
if (wordpos(RID,MoreIDs) >< 0) then  /* if the code is in the list */
do until (Line.LineCount = '.')  /* go on until we find a full stop */
 LineCount = LineCount + 1  /* up the counter */
 Line.LineCount = GetResponseLine(Sock)  /* get the next line */
end

return ''  /* end of GetResponse */

/**********************************************************************/
GetResponseLine: procedure expose !.  /* get a line from the server */
/**********************************************************************/

parse arg Sock  /* get the argument */

CRLF = d2c(13)||d2c(10)  /* define CRLF */

if (symbol('!.Buff') = 'LIT') then  /* if the symbol in the buffer is a literal */
do
 !.Buff = ''  /* empty the buffer */
end

do while (pos(CRLF,!.Buff) = 0)  /* as long as we have no CRLF in the buffer */
 TRC = SockRecv(Sock,'Data',8000)  /* get data from the socket */
 !.Buff = !.Buff||Data  /* add the results to the buffer */
end

parse var !.Buff Line (CRLF) !.buff  /* take the bit before the CRLF from the buffer */

return Line  /* end of GetResponseLine */

/**********************************************************************/
SendMessage: procedure expose !.  /* send a string to the server */
/**********************************************************************/

parse arg Sock,Data  /* get the arguments */
Data = Data||d2c(13)||d2c(10)  /* add a CRLF to the data */
Len = length(Data)  /* calculate the length of the data */
do while (Len > 0)  /* as long as we have something left */
 Len = SockSend(Sock,Data);  /* send data to the socket */
 if (Len <= 0) then  /* if the socket returns a negative value, it is dead */
 do
  say 'Server closed the connection'  /* report */
 end
 Data = substr(Data,Len+1)  /* lose the first character of the data */
 Len  = length(Data)  /* recalculate the length of the data */
end

return 0  /* end of SendMessage */

/**********************************************************************/
LoadRexxUtils: procedure  /* loads REXX utility functions if necessary */
/**********************************************************************/

Success = 1  /* all is well to start with */

if (rxfuncquery('SysLoadFuncs') >< 0) then  /* if we have to load the REXX utility functions */
do
 if (rxfuncadd('SysLoadFuncs','RexxUtil','SysLoadFuncs') = 0) then  /* if we can register the general loading function */
 do
  call sysloadfuncs  /* call the general loading function */
  say 'Rexx utility functions loaded'  /* report */
 end
 else  /* if we cannot register the general loading function */
 do
  say  'Cannot load Rexx utility functions'  /* report */
  Success = 0  /* no success */
 end
end

return Success  /* end of LoadRexxUtils */

/**********************************************************************/
LoadRexxSocks: procedure  /* loads REXX socket functions if necessary */
/**********************************************************************/

Success = 1  /* all is well to start with */

if (rxfuncquery('SockLoadFuncs') >< 0) then  /* if we have to load the socket functions */
do
 if (rxfuncadd('SockLoadFuncs','RxSock','SockLoadFuncs') = 0) then  /* if we can register the general loading function */
 do
  call sockloadfuncs  /* call the general loading function */
  say 'Rexx socket functions loaded'  /* report */
 end
 else  /* if we cannot register the general loading function */
 do
  say  'Cannot load Rexx socket functions'  /* report */
  Success = 0  /* no success */
 end
end

return Success  /* end of LoadRexxUtils */

/**********************************************************************/
Halt:  /* handles halt condition */
/**********************************************************************/

exit Messages  /* just quit */
