/**********************************************************************/
/*                                                                    */
/* RexxMail is a REXX program that uses the Workplace Shell, the      */
/* graphical user interface of the IBM OS/2 operating system, to      */
/* create an easy to use, object-oriented, highly flexible e-mail     */
/* message processing system.                                         */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program forms part of the RexxMail package, and may not be    */
/* distributed separately.                                            */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use RexxComm.CMD to reformat the comments in this program.         */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* The latest versions of RexxMail and RexxComm can be found at       */
/*               www.degeus.com/rexx                                  */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* This program is released under the terms of the GNU license, see   */
/*               www.gnu.org/copyleft/gpl.html                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* (c) 2000-2001 Marcus C. de Geus                                    */
/*               marcus@degeus.com                                    */
/*               www.degeus.com                                       */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Use it if you like it. Don't if you don't. No legalese.            */
/*                                                                    */
/**********************************************************************/

/**********************************************************************/
/* Main                                                               */
/**********************************************************************/

signal on Halt  /* handles halt condition */
signal on Syntax  /* handles syntax errors, i.e. invalid REXX versions */

drop Global.  /* start with nothing */

/**********************************************************************/
/* Get the operating system, program directory and program name       */
/**********************************************************************/

parse source OS . ProgSpec  /* get the OS and ProgSpec */

Global.ProgDir = filespec('D',ProgSpec)||filespec('P',ProgSpec)  /* the program directory */
Global.ProgName = filespec('N',ProgSpec)  /* the program name */

/**********************************************************************/
/* initialize some global variables                                   */
/**********************************************************************/

Global.Build = 'RexxMail - build 20010401.105640 - (c) 2001 Marcus C. de Geus'  /* define the build */

say Global.Build  /* report it */

Global.ConfDir = Global.ProgDir  /* use the program directory until we get the correct location (we may need to write an error log file here) */

Global.Error = ''  /* no error messages yet */

Global.Signals = 1  /* start with signals on to catch errors before the settings file is read */

Global.DispatchReply = 0  /* assume any switch was processed in the standard way */

/**********************************************************************/
/* initialize global strings                                          */
/**********************************************************************/

Global.CRLF = d2c(13)||d2c(10)  /* define CRLF */

Global.EmptyLine = Global.CRLF||Global.CRLF  /* define an empty line */

Global.Warning = ''  /* the warning string to use */

Global.ESCtoASC = '_________'||d2c(9)||d2c(10)||'__'||d2c(13)||'__'||,  /* bytes 00 to 0F of the escape characters => ASCII conversion table */
                  '________________'  /* bytes 10 to 1F */

Global.850toASC = 'CueaaaaceeeiiiAA'||,  /* bytes 80 to 8F of the PC850 => ASCII conversion table */
                  'EaAooouuyOUoLOxf'||,  /* bytes 90 to 9F */
                  'aiounNao?r___!__'||,  /* bytes A0 to AF */
                  '_____AAAc____cY_'||,  /* bytes B0 to BF */
                  '______aA_______*'||,  /* bytes C0 to CF */
                  'dDEEEeIII____|I_'||,  /* bytes D0 to DF */
                  "O_OOoO___UUUyY_'"||,  /* bytes E0 to EF */
                  '-_____/__"-132*_'  /* bytes F0 to FF */

Global.850toISO = ''||,  /* bytes 80 to 8F of the PC850 => ISO-8859-1 conversion table */
                  'f'||,  /* bytes 90 to 9F */
                  'Ѫ'||,  /* bytes A0 to AF */
                  '__________'||,  /* bytes B0 to BF */
                  '_____________'||,  /* bytes C0 to CF */
                  'Ȁ_____'||,  /* bytes D0 to DF */
                  'յݯ'||,  /* bytes E0 to EF */
                  '_*'  /* bytes F0 to FF */

Global.ISOto850 = '_______________'||,  /* bytes 80 to 8F of the ISO-8859-1 => PC850 conversion table */
                  '________________'||,  /* bytes 90 to 9F */
                  'Ͼ'||,  /* bytes A0 to AF */
                  ''||,  /* bytes B0 to BF */
                  'ǎԐ'||,  /* bytes C0 to CF */
                  'ѥ噞'||,  /* bytes D0 to DF */
                  'Ƅ'||,  /* bytes E0 to EF */
                  'Ф'  /* bytes F0 to FF */

/**********************************************************************/
/* parse the command line arguments                                   */
/**********************************************************************/

parse upper arg PreArg '/' Switch Argument  /* get the arguments */

if (PreArg >< '') then  /* if there is an argument before the switch */
do
 call AddError 'Invalid input'  /* report */
 call Halt  /* and quit */
end

if (Switch = '') then  /* if we have no switch */
do
 call AddError 'Missing switch'  /* report */
 call Halt  /* and quit */
end

if (Switch = 'HELP') then  /* if the switch was "help" */
do
 call HelpText  /* call the help message procedure */
 call Halt  /* and quit */
end

/**********************************************************************/
/* Make sure this is OS/2                                             */
/**********************************************************************/

if (OS >< 'OS/2') then  /* if the OS is not OS/2 */
do
 call AddError 'This program requires OS/2'  /* report */
 call Halt  /* and quit */
end

/**********************************************************************/
/* Try to load the REXX utilities if necessary                        */
/**********************************************************************/

if (LoadRexxUtil() = 2) then  /* if we cannot load the Rexxx utilities library */
do
 call AddError 'Cannot load the REXX utilities library'  /* report */
 call Halt  /* and quit */
end

/**********************************************************************/
/* Try to load code page 850                                          */
/**********************************************************************/

Global.OriginalCP = sysqueryprocesscodepage()  /* get the current code page */

if (syssetprocesscodepage(850) >< 0) then  /* if we cannot load code page 850 */
do
 call AddError 'Cannot load code page 850'  /* report */
 call Halt  /* and quit */
end

/**********************************************************************/
/* initialize global directory variables based on Global.ProgDir      */
/**********************************************************************/

Global.TempDir = Global.ProgDir||'TEMP'  /* the RexxMail temp directory */
if (\CheckDir(Global.TempDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

Global.IconDir = Global.ProgDir||'ICONS'  /* the RexxMail icon directory */
if (\CheckDir(Global.IconDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

Global.InfoDir = Global.ProgDir||'INFORMATION'  /* the RexxMail information directory, which need not exist */

/**********************************************************************/
/* Find the mail files location                                       */
/**********************************************************************/

LocationFile = Global.ProgDir||'LOCATION.TXT'  /* the location file */

if (stream(LocationFile,'C','QUERY EXISTS') = '') then  /* if we cannot find a location file */
do
 call AddError 'Could not find : '||LocationFile  /* report */
 call Halt  /* and quit */
end

User = value('USER',,'OS2ENVIRONMENT')  /* get the user name */
if (User >< '') then  /* if we have a user name */
do
 say 'User        = "'||User||'"'  /* report */
 Global.MainDir = GetFileEntry(LocationFile,User)  /* look for the user's location in the location file */
end
else  /* if we have no user name */
do
 say 'No user name found in environment'  /* report */
 Global.MainDir = ''  /* we have no location yet */
end

if (Global.MainDir = '') then  /* if we have no directory spec */
do
 say 'No user location entry found; looking for default'  /* report */
 Global.MainDir = GetFileEntry(LocationFile,'DEFAULT')  /* look for the default location in the location file */
end

if (Global.MainDir = '') then  /* if we have no directory spec */
do
 call AddError 'No user location defined for "'||UserName||'"'||Global.CRLF,  /* report */
               'and no default main directory found in "'||LocationFile||'"'  /* report */
 call Halt  /* quit */
end

Global.MainDir = translate(Global.MainDir)  /* make it uppercase */

if (\CheckDir(Global.MainDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

say 'Main folder = "'||Global.MainDir||'"'  /* report */

/**********************************************************************/
/* initialize global directory variables based on Global.MainDir      */
/**********************************************************************/

Global.ConfDir = Global.MainDir||'\CONFIGURATION'  /* the configuration directory */
if (\CheckDir(Global.ConfDir)) then  /* if the directory does not exist */
do
 Global.ConfDir = ''  /* use nothing */
 call Halt  /* quit */
end

Global.InDir = Global.MainDir||'\IN'  /* the mail in folder */
if (\CheckDir(Global.InDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

Global.OutDir = Global.MainDir||'\OUT'  /* the mail out folder */
if (\CheckDir(Global.OutDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

Global.AddrDir = Global.MainDir||'\ADDRESSES'  /* the address book folder */
if (\CheckDir(Global.AddrDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

Global.InArchDir = Global.MainDir||'\IN ARCHIVE'  /* the in archive folder, which need not exist */
Global.OutArchDir = Global.MainDir||'\OUT ARCHIVE'  /* the out archive folder, which need not exist */

/**********************************************************************/
/* get more global variables from the settings file                   */
/**********************************************************************/

SettingsEntries = 'ADDRESS',  /* defines the default e-mail address of the sender for outgoing messages */
                  'ASCIILINELENGTH',  /* ASCIILineLength determines the maximum length of ASCII message lines; defaults to 72 */
                  'ASCIITEXT',  /* ASCIIText = YES causes RexxMail to convert outgoing messages to ASCII, word-wrapped at ASCIILineLength characters */
                  'CLEANUPTEMP',  /* CleanupTemp = YES causes RexxMail to delete unused files and folders from the Temp dir when the /Open switch is used */
                  'EDITOR',  /* defines the command to run on editing outgoing messages */
                  'ISODATES',  /* ISODates = YES causes RexxMail to rewrite dates in ISO notation */
                  'MAILLOGS',  /* MailLogs = YES causes RexxMail to log dispatched and registered messages */
                  'NAME',  /* defines the default "real" name of the sender for outgoing messages */
                  'NAMES83',  /* defines the extensions of file names that will be reduced to 8+3 characters */
                  'OBJECTTITLE',  /* defines the parts to be included in mail message object titles */
                  'READER',  /* defines the command to run for reading the text part of incoming messages */
                  'READERHEADER',  /* defines which message header entries of incoming messages are to be included in the reader file */
                  'RUNATTACHOUT',  /* defines the command to run on outgoing attachments before they are added to the sendable message file */
                  'RUNATTACHIN',  /* defines the command to run on incoming mail attachments right after they have been unpacked from a mesage file */
                  'RUNDISPATCH',  /* defines the command to run on sendable message files */
                  'SIGNALS',  /* Signals = YES causes RexxMail to sound signals after certain operations */
                  'STATIONERY',  /* defines the name of the stationery folder */
                  'TITLEADDRESS',  /* TitleAddress = Name | Address | Full */
                  'WARNICON'  /* WarnIcon = YES causes RexxMail to create a warning icon on the desktop if mail comes in while the main mail window is closed */

WordNo = 0  /* start at 0 */
do words(SettingsEntries)  /* for each word in the list */
 WordNo = WordNo + 1  /* up the counter */
 VarName = subword(SettingsEntries,WordNo,1)  /* get the variable name from the list */
 Global.VarName = ''  /* initialize the global variable of that name */
end

SettingsFile = Global.ConfDir||'\SETTINGS.TXT'  /* the RexxMail settings file name */

if (OpenFile(SettingsFile)) then  /* if we can open the RexxMail settings file */
do
 do while (lines(SettingsFile))  /* go on until we run out of lines */
  Line = linein(SettingsFile)  /* read the next line */
  Line = translate(Line,d2c(32),d2c(9))  /* convert TABs to spaces */
  Line = strip(Line,'B',' ')  /* get rid of excess spaces */
  if (Line >< '') then  /* if the line is not empty */
  do
   if (substr(Line,1,1) >< '#') then  /* if it is not a comment */
   do
    Line = translate(Line,d2c(32),d2c(9))  /* turn TABs into spaces */
    parse var Line SettingsEntry '=' SettingsValue  /* get the parts we need */
    SettingsEntry = translate(strip(SettingsEntry))  /* get rid of superfluous spaces and make it upper case so the variable tail will work */
    if (wordpos(SettingsEntry,SettingsEntries) > 0) then  /* if it is one of the range of possible entries */
    do
     Global.SettingsEntry = strip(SettingsValue)  /* get rid of superfluous spaces and use the value */
    end
    else  /* if it is not a valid entry */
    do
     call AddError 'Invalid entry in settings file "'||SettingsEntry||'"'  /* report */
    end
   end
  end
 end
 call stream SettingsFile,'C','CLOSE'  /* close the RexxMail settings file */
end

drop SettingsEntries  /* clear up */

/**********************************************************************/
/* Set default values for any missing Boolean global variables        */
/**********************************************************************/

Global.ASCIIText = (translate(Global.ASCIIText) = 'YES')  /* ASCIIText is true if the settings file entry says yes, i.e. it defaults to untrue  */
Global.ISODates = (translate(Global.ISODates) = 'YES')  /* ISODates is true if the settings file entry says yes, i.e. it defaults to untrue  */
Global.MailLogs = (translate(Global.MailLogs) = 'YES')  /* MailLogs is true if the settings file entry says yes, i.e. it defaults to untrue  */
Global.WarnIcon = (translate(Global.WarnIcon) = 'YES')  /* WarnIcon is true if the settings file entry says yes, i.e. it defaults to untrue */
Global.Signals = (translate(Global.Signals) = 'YES')  /* Signals is true if the settings file entry says yes, i.e. it defaults to untrue  */
Global.CleanupTemp = (translate(Global.CleanupTemp) = 'YES')  /* CleanupTemp is true if the settings file entry says yes, i.e. it defaults to untrue  */

/**********************************************************************/
/* Set default values for any other missing global variables          */
/**********************************************************************/

if (Global.Editor = '') then  /* if no editor is defined */
do
 Global.Editor = 'e.exe "%N"'  /* use e.exe */
end

if (Global.Reader = '') then  /* if no reader is defined */
do
 Global.Reader = 'e.exe "%N"'  /* use e.exe */
end

if (datatype(Global.ASCIILineLength,'W')) then  /* if ASCIILineLength is a whole number */
do
 if (Global.ASCIILineLength = 0) then  /* if it is 0 */
 do
  Global.ASCIILineLength = 72  /* make it 72 */
 end
end
else  /* if it is not a whole number */
do
 Global.ASCIILineLength = 72  /* make it 72 */
end

if (Global.ObjectTitle = '') then  /* if we have no entry for the object title strings */
do
 Global.ObjectTitle = 'DATE SENDER RECIPIENTS SUBJECT'  /* use this as the default value */
end

if (Global.Stationery = '') then  /* if we have no stationery entry */
do
 Global.Stationery = 'blue_white'  /* use this */
end

Global.PaperDir = Global.IconDir||'\'||Global.Stationery  /* the stationery directory to use */
if (\CheckDir(Global.PaperDir)) then  /* if the directory does not exist */
do
 call Halt  /* quit */
end

/**********************************************************************/
/* Process the command-line argument                                  */
/**********************************************************************/

FileSwitches = 'EDIT',  /* a switch that requires a file spec */
               'RAW',  /* a switch that requires a file spec */
               'REGISTER',  /* a switch that requires a file spec */
               'REPLY',  /* a switch that requires a file spec */
               'REPLYTOALL',  /* a switch that requires a file spec */
               'REPLYTORECIPIENTS',  /* a switch that requires a file spec */
               'REPLYTOSENDER',  /* a switch that requires a file spec */
               'SETASCII',  /* a switch that requires a file spec */
               'SETNONASCII',  /* a switch that requires a file spec */
               'SETDISPATCHED',  /* a switch that requires a file spec */
               'SETNEW',  /* a switch that requires a file spec */
               'SETRECEIVED',  /* a switch that requires a file spec */
               'SETVIEWED',  /* a switch that requires a file spec */
               'VIEW'  /* a switch that requires a file spec */

NoFileSwitches = 'EXPORT',  /* a switch that does not take an argument */
                 'IMPORT',  /* a switch that does not take an argument */
                 'INFO',  /* a switch that does not take an argument */
                 'OPEN'  /* a switch that does not take an argument */

if (Argument = '') then  /* if we have no argument */
do
 if (Switch = 'DISPATCH') then  /* if this is the switch */
 do
  Argument = Global.OutDir||'\*.*'  /* use the files in the out folder */
 end
 else  /* if it is not that switch */
 do
  if (wordpos(Switch,FileSwitches) > 0) then  /* if the switch is one that needs a file spec */
  do
   call AddError Switch||' requires a filespec'  /* report */
   call Halt  /* and quit */
  end
 end
end
else  /* if we have an argument */
do
 if (wordpos(Switch,NoFileSwitches) > 0) then  /* if the switch is one that does not take a file spec */
 do
  call AddError Switch||' does not take an argument'  /* report */
  call Halt  /* and quit */
 end
end

if (Argument = '') then  /* if we have no argument */
do
 Global.Filespec.0 = 1  /* we have no files to process, but we'll run the switch call once */
 Global.Filespec.1 = ''  /* use no argument */
end
else  /* if we have an argument */
do
 Argument = strip(Argument,'B',' ')  /* get rid of excess spaces */
 Argument = strip(Argument,'B','"')  /* get rid of any quotes */
 call sysfiletree Argument,'Global.Filespec.','FO'  /* look for a file of this name */
 if (Global.Filespec.0 = 0) then  /* if the filespec was not found */
 do
  if (wordpos(Switch,FileSwitches) > 0) then  /* if the switch is one of these */
  do
   call AddError Argument||' : no files found'  /* report an error */
  end
  else  /* if finding no files to work on is not a fatal error */
  do
   say Argument||' : no files found'  /* simply report */
  end
  call Halt  /* and quit */
 end
end

/**********************************************************************/
/* process the switch                                                 */
/**********************************************************************/

select  /* do one of the following */

 when (Switch = 'ADDRESS') then  /* if the switch is "address" */
 do
  if (Global.Filespec.1 >< '') then  /* if we have a file spec */
  do Index = 1 to Global.Filespec.0  /* take each of the files */
   call MakeAddressEntry Global.Filespec.Index  /* call the address book procedure */
  end
  if (\sysopenobject(Global.AddrDir,0,1)) then  /* if we cannot open the address book folder */
  do
   call AddError 'Cannot open the address book folder'  /* report */
  end
 end

 when (Switch = 'DISPATCH') then  /* if the switch is  "dispatch" */
 do Index = 1 to Global.Filespec.0  /* take each of the files in FileSpec */
  if (MessageSettings(Global.Filespec.Index,'+*******','Match')) then  /* if the file is an incoming message */
  do
   call MakeReplyMessage Global.Filespec.Index  /* generate a reply message */
   Global.DispatchReply = 1  /* the switch was processed in a non-standard way */
  end
  else  /* if the file is an outgoing message */
  do
   call DispatchMessage Global.Filespec.Index  /* dispatch the message */
  end
 end

 when (Switch = 'EDIT') then  /* if the switch is  "edit" */
 do Index = 1 to Global.Filespec.0  /* take each of the files in FileSpec */
  call EditMessage Global.Filespec.Index  /* call the message edit procedure */
 end

 when (Switch = 'EXPORT') then  /* if the switch is "export */
 do
  call AddressExport  /* call the mail address book export procedure */
 end

 when (Switch = 'IMPORT') then  /* if the switch is "import" */
 do
  call AddressImport  /* call the mail address book import procedure */
 end

 when (Switch = 'INFO') then  /* if the switch is "info" */
 do
  call OpenInfo  /* call the info display procedure */
 end

 when (Switch = 'OPEN') then  /* if the switch is "open" */
 do
  call OpenFolders  /* call the open mail folders procedure */
 end

 when (Switch = 'RAW') then  /* if the switch is "raw" */
 do Index = 1 to Global.Filespec.0  /* take each of the files in FileSpec */
  call ViewRawMessage Global.Filespec.Index  /* call the raw message viewing procedure */
 end

 when (Switch = 'REGISTER') then  /* if the switch is "register" */
 do Index = 1 to Global.Filespec.0  /* take each of the files in FileSpec */
  call RegisterMessage Global.Filespec.Index  /* call the mail register procedure */
 end

 when (substr(Switch,1,5) = 'REPLY') then  /* if the switch starts with "reply" */
 do Index = 1 to Global.Filespec.0  /* take each of the files in FileSpec */
  call MakeReplyMessage Global.Filespec.Index,SWITCH  /* call the message reply procedure with the switch as argument */
 end

 when (substr(Switch,1,3) = 'SET') then  /* if the switch starts with "set" */
 do Index = 1 to Global.Filespec.0  /* take each of the files in FileSpec */
  call SetMessageType Global.Filespec.Index,Switch  /* call the message settings procedure with the switch as argument */
 end

 when (Switch = 'VIEW') then  /* if the switch was "view" */
 do Index = 1 to Global.Filespec.0  /* take each of the files in FileSpec */
  call ViewMessage Global.Filespec.Index  /* call the message viewing procedure */
 end

 otherwise  /* if none of the above applies */
 do
  call AddError 'Invalid switch "'||Switch||'"'  /* report */
 end

end

call Halt  /* that's all, folks! */

/**********************************************************************/
LoadRexxUtil: procedure expose Global.  /* try to load the REXX utilities library */
/**********************************************************************/

if (rxfuncquery('SysLoadFuncs')) then  /* if it is not already loaded */
do
 if (rxfuncadd('SysLoadFuncs','RexxUtil','SysLoadFuncs') = 0) then  /* if we can add it */
 do
  if (SysLoadFuncs() = 0) then  /* if we cannot load it */
  do
   Result = 2  /* bad result */
  end
  else  /* if we can load it */
  do
   Result = 1  /* indicate we loaded the library */
  end
 end
end
else  /* if the library had already been loaded */
do
 Result = 0  /* indicate that the REXX utilities library had already been loaded */
end

return Result  /* end of LoadRexxUtil */

/**********************************************************************/
CheckDir: procedure expose Global.  /* checks the existence of a directory */
/**********************************************************************/

parse arg DirSpec  /* get the argument */

Success = 0  /* no success to begin with */

CurDir = directory()  /* store the current dir */

if (directory(DirSpec) = '') then  /* if we cannot change to the directory */
do
 call AddError 'Cannot find directory "'||DirSpec||'"'  /* report */
end
else  /* if we can change to the directory */
do
 if (directory(CurDir) = '') then  /* if we cannot change back to the original directory */
 do
  call AddError 'Cannot return to original directory: "'||CurDir||'"'  /* report */
 end
 else  /* if we can change back to the original directory */
 do
  Success = 1  /* all is well */
 end
end

return Success  /* end of CheckDir */

/**********************************************************************/
OpenFile: procedure expose Global.  /* attempts to open a file */
/**********************************************************************/

parse arg FileSpec,Operation  /* get the arguments */

Attempts = 0  /* we haven't tried yet */

do until ((Success) | (Attempts = 10))  /* go on until we succeed or reach the maximum no. of attempts */
 Success = (stream(FileSpec,'C','OPEN '||Operation) = 'READY:')  /* if we can open the file, all is well */
 if (\Success) then  /* if we did not succeed */
 do
  Attempts = Attempts + 1  /* up the attempts counter */
  if (Attempts = 10) then  /* if we've reached the last try */
  do
   call AddError 'Cannot open file : '||FileSpec  /* report */
  end
  else  /* if we have more tries left */
  do
   call syssleep 0.1  /* wait a bit */
  end
 end
end

return Success  /* end of OpenFile */

/**********************************************************************/
AddError: procedure expose Global.  /* add an error string*/
/**********************************************************************/

parse arg Message  /* get the argument */

Global.Error = Global.Error||Global.CRLF||Message  /* add the message to the existing error string */

return  /* end of AddError */

/**********************************************************************/
Syntax:  /* handles syntax errors */
/**********************************************************************/

call AddError 'REXX did not recognize the instruction on line '||,  /* report */
              Sigl||':'||Global.CRLF||'    '||sourceline(Sigl)  /* report */

call Halt  /* and quit */

/**********************************************************************/
Halt:  /* handles halt condition */
/**********************************************************************/

if (symbol(Global.OriginalCP) = 'VAR') then  /* if the original code page is defined */
do
 call syssetprocesscodepage Global.OriginalCP  /* restore the original code page */
end

Errors = (Global.Error >< '')  /* do we have errors? */

if (Errors) then  /* if we do */
do

 if (Global.Signals) then  /* if a signal is wanted */
 do
  call beep 888,100  /* signal */
  call beep 111,200  /* signal */
 end

 if (Global.ConfDir >< '') then  /* if we have a configuration dir */
 do
  ErrorLog = Global.ConfDir||'\errorlog.txt'  /* the name of the error log file */
  if (OpenFile(ErrorLog,'WRITE')) then  /* if we can open the file for writing */
  do
   DTStamp = substr(date('S'),1,4)||'-'||,  /* start the date & time stamp with the year in 4 digits */
             substr(date('S'),5,2)||'-'||,  /* followed by the month in 2 digits */
             substr(date('S'),7,2)||' - '||,  /* followed by the day of the month in 2 digits */
             time('N')||' :'  /* followed by the normal time */
   call lineout ErrorLog,DTStamp||Global.Error  /* write the error text to the log file */
   call stream ErrorLog,'C','CLOSE'  /* close the log file */
  end
 end

 say Global.Error  /* send the error text to standard output */
 say ''  /* add an empty line */
 say 'Type RexxMail /help for a list of available options'  /* report */

end

ResultCode = Errors + (Global.DispatchReply * 2)  /* add 2 to the result code if a drop action on the "Dispatch" object resulted in a reply message */

exit ResultCode  /* quit */

/**********************************************************************/
AddressExport: procedure expose Global.  /* exports address book contents and edits address text file */
/**********************************************************************/

ExpFile = Global.ConfDir||'\mailaddr.txt'  /* the address file to edit */

say 'Exporting address data to : '||translate(ExpFile)  /* report */

BakName = 'mailaddr.bak'  /* the backup address file name */
BakFile = Global.ConfDir||'\'||BakName  /* the full backup address file spec */
if (stream(ExpFile,'C','QUERY EXISTS') >< '') then  /* if the old address file already exists */
do
 if (stream(BakFile,'C','QUERY EXISTS') >< '') then  /* if the backup file already exists */
 do
  if (sysfiledelete(BakFile) > 0) then  /* if we cannot delete the old backup file */
  do
    call AddError 'Cannot delete "'||BakFile||'"'  /* report */
    return 0  /* and quit */
  end
 end
 if (\syssetobjectdata(ExpFile,'TITLE='||BakName||';')) then  /* if we cannot rename the old address file */
 do
  call AddError 'Cannot rename "'||ExpFile||'"'  /* report */
  return 0  /* and quit */
 end
end

if (\GetAddData(ExpFile)) then  /* if we cannot get the data from the template objects in the address folder */
do
 call AddError 'Cannot get address folder data'  /* report */
 return 0  /* and quit */
end

return 1  /* that's all, folks! */

/**********************************************************************/
GetAddData: procedure expose Global.  /* collects template date from the address folder */
/**********************************************************************/

parse arg ExpFile  /* get the argument */

if (\OpenFile(ExpFile,'WRITE')) then  /* if we cannot open the address file for writing */
do
 return 0  /* and quit */
end

OutText = ''  /* start with nothing */

call sysfiletree Global.AddrDir||'\*.*','Files.','FOS'  /* get the address book contents */

do Index = 1 to Files.0  /* take each of the objects */
 if (\OpenFile(Files.Index,'READ')) then  /* try to open the file for reading, and if we cannot do so */
 do
  return 0  /* and quit */
 end

 Lines = charin(Files.Index,1,chars(Files.Index))  /* store the contents of the file in an input buffer */

 call stream Files.Index,'C','CLOSE'  /* close the file */

 parse var Files.Index (Global.AddrDir) '\' Name  /* get the bit of the file spec we want */
 OutText = OutText||'<TITLE>'||Global.CRLF||Name||Global.CRLF  /* add the file name to the output buffer with a header */
 if (Lines >< '') then  /* if we have lines */
 do
  OutText = OutText||'<TEXT>'||Global.CRLF||Lines||Global.CRLF  /* add them to the output buffer with a header */
 end
 KeyPhrases = GetKeyPhrases(Files.Index)  /* get the file's key phrases, if any */
 if (KeyPhrases >< '') then  /* if we have something */
 do
  OutText = OutText||'<KEYWORDS>'||Global.CRLF||KeyPhrases||Global.CRLF  /* add it to the output buffer with a header */
 end
 OutText = OutText||'<END>'||Global.CRLF||Global.CRLF  /* add an ending to the output buffer, and an empty line */
end

call lineout ExpFile,'This text file describes the contents of the RexxMail Addresses folder.'  /* write a line of explanatory text to the file */
call lineout ExpFile,'It can be created by means of the RexxMail "Addresses - Export " option,'  /* write a line of explanatory text to the file */
call lineout ExpFile,'and it can be used to recreate the Addresses Folder contents by means of'  /* write a line of explanatory text to the file */
call lineout ExpFile,'the "Addresses - Import " function.'  /* write a line of explanatory text to the file */
call lineout ExpFile,''  /* write an empty line to the file */
call lineout ExpFile,'Each mail message template object in the Addresses folder or one of its'  /* write a line of explanatory text to the file */
call lineout ExpFile,'subfolders is represented by a group of text lines subdivided by <...>'  /* write a line of explanatory text to the file */
call lineout ExpFile,'tags separating the sections of data making up the object contents.'  /* write a line of explanatory text to the file */
call lineout ExpFile,'A group of lines must start with a <TITLE> line and end with an <END>'  /* write a line of explanatory text to the file */
call lineout ExpFile,'line. Text on the line(s) following the <TITLE> tag and preceding the'  /* write a line of explanatory text to the file */
call lineout ExpFile,'next tag will be used for the title of the mail message template object.'  /* write a line of explanatory text to the file */
call lineout ExpFile,'Titles are relative to the RexxMail Addresses folder; simply include'  /* write a line of explanatory text to the file */
call lineout ExpFile,'backslash markers to create subfolders.'  /* write a line of explanatory text to the file */
call lineout ExpFile,''  /* write an empty line to the file */
call lineout ExpFile,'The other possible tags are <TEXT> and <KEYWORDS>. Text on the line(s)'  /* write a line of explanatory text to the file */
call lineout ExpFile,'following the <TEXT> tag and preceding the next tag will be written as'  /* write a line of explanatory text to the file */
call lineout ExpFile,'text content to the message file. Text on the line(s) following the'  /* write a line of explanatory text to the file */
call lineout ExpFile,'<KEYWORDS> tag and preceding the next tag (which must be the <END> tag)'  /* write a line of explanatory text to the file */
call lineout ExpFile,'will be written to the "Keywords" Extended Attribute field of the'  /* write a line of explanatory text to the file */
call lineout ExpFile,'message file (on the "File" page in the "Properties" notebook).'  /* write a line of explanatory text to the file */
call lineout ExpFile,''  /* write an empty line to the file */
call lineout ExpFile,'Tags within a text line block must appear in the following order:'  /* write a line of explanatory text to the file */
call lineout ExpFile,'  <TITLE>'  /* write a line of explanatory text to the file */
call lineout ExpFile,'  <TEXT>'  /* write a line of explanatory text to the file */
call lineout ExpFile,'  <KEYWORDS>'  /* write a line of explanatory text to the file */
call lineout ExpFile,'  <END>'  /* write a line of explanatory text to the file */
call lineout ExpFile,'________________________________________________________________________'  /* write a line to the file */

call lineout ExpFile,''  /* write an empty line to the file */

call charout ExpFile,OutText  /* write the contents of the output buffer to the file */

call stream ExpFile,'C','CLOSE'  /* close the file */

return 1  /* end of GetAddressData */

/**********************************************************************/
GetKeyPhrases: procedure expose Global.  /* gets key phrases from an address book entry */
/**********************************************************************/

parse arg Entry  /* get the argument */

OutLines = ''  /* nothing yet */

call sysgetea Entry,'.KEYPHRASES','KeyPhrases'  /* look for key phrases */
if (KeyPhrases >< '') then  /* if we found key phrases */
do
 parse var KeyPhrases EAT 3 CodePage 5 Count 7 Contents  /* get the bits we want */
 if (EAT = reverse('FFDF'x)) then  /* if it is a multiple value, multiple type entry */
 do
  Count = c2d(reverse(Count))  /* extract the number of keyword lines */
  do Index = 1 to Count  /* take each line entry */
   parse var Contents EAT 3 Len 5  /* get the bits we want */
   if (EAT = reverse('FFFD'x)) then  /* it it's ASCII text (which it should be) */
   do
    Length = c2d(reverse(Len))  /* get the length of the next line */
    parse var Contents 5 Text +(Length) Contents  /* get the bits we want */
    if (Length > 0) then  /* if we have something */
    do
     OutLines = OutLines||Text||Global.CRLF  /* add a CRLF and Text */
    end
   end
  end
 end
end

if (OutLines >< '') then  /* if we found something */
do
 OutLines = substr(OutLines,1,length(OutLines)-2)  /* get rid of the excess CRLF at the end */
end

return OutLines  /* end of GetKeyPhrases */

/**********************************************************************/
AddressImport: procedure expose Global.  /* imports address book entries */
/**********************************************************************/

ImpFile = Global.ConfDir||'\mailaddr.txt'  /* use the standard file name */

say 'Importing address data from : '||translate(ImpFile)  /* report */

if (stream(ImpFile,'C','QUERY EXISTS') = '') then  /* if the address text file does not exist */
do
 call AddError 'Cannot find "'||ImpFile||'"'  /* report */
 return 0  /* and quit */
end

if (\PutAddData(ImpFile,Global.AddrDir)) then  /* if we cannot put the template data from the emergency file into the address folder */
do
 call AddError 'An error occurred while writing the address folder data'  /* report */
end

return 1  /* that's all, folks! */

/**********************************************************************/
PutAddData: procedure expose Global.  /* puts the address data from the edit file into the template objects in the address folder */
/**********************************************************************/

parse arg EditFile,AddDir  /* get the names of the edit ffile and the address folder */

if (\OpenFile(EditFile,'READ')) then  /* if we cannot open the address file for writing */
do
 return 0  /* and quit */
end

call sysfiletree AddDir||'\*.*','Objects','BO'  /* get the contents */

do Index = 1 to Objects.0  /* take each of the file objects */
 if (\sysdestroyobject(Objects.Index)) then  /* if we cannot get rid of the object */
 do
  call AddError 'Cannot delete "'||Objects.Index||'"'  /* report */
 end
end

InBuffer = charin(EditFile,1,chars(EditFile))  /* read the contents of the address file into an input buffer */
InBuffer = Global.CRLF||InBuffer||Global.CRLF  /* add a CRLF at both ends to make sure the block markers are recognized */
call stream EditFile,'C','CLOSE'  /* close the file */

Sep_1 = Global.CRLF||'<TITLE>'||Global.CRLF  /* define a separator */
Sep_2 = Global.CRLF||'<TEXT>'||Global.CRLF  /* define a separator */
Sep_3 = Global.CRLF||'<KEYWORDS>'||Global.CRLF  /* define a separator */
Sep_4 = Global.CRLF||'<END>'||Global.CRLF  /* define a separator */

LFCR = d2c(10)||d2c(13)  /* define a reverse CRLF */

EntryNo = 0  /* start at 0 */

do while (InBuffer >< '')  /* go on while we have contents left */
 EntryNo = EntryNo + 1  /* up the entry number */
 parse var InBuffer (Sep_1) Block (Sep_4) InBuffer  /* get the next block */
 if (Block = '') then  /* if the block is empty */
 do
  if (InBuffer >< '') then  /* if it not because we have run out of contents */
  do
   call AddError 'Entry number '||EntryNo||' contains no data'  /* report */
  end
 end
 else  /* if the block is not empty */
 do
  Name = ''  /* start with nothing */
  Text = ''  /* start with nothing */
  Comm = ''  /* start with nothing */
  parse var Block Block (Sep_3) Comm  /* extract any keywords */
  parse var Block Block (Sep_2) Text  /* extract any text content */
  Name = Block  /* the rest must be a name */
  do while (substr(Name,1,2) = Global.CRLF)  /* as long as the name starts with a CRLF */
   Name = substr(Name,3)  /* get rid of it */
  end
  RevName = reverse(Name)  /* reverse the name */
  do while (substr(RevName,1,2) = LFCR)  /* as long as the reverse name starts with a reverse CRLF */
   RevName = substr(RevName,3)  /* get rid of it */
  end
  Name = reverse(RevName)  /* turn the name back again */
  if (Name = '') then  /* if we have no name */
  do
   call AddError 'Entry number '||EntryNo||' contains no name'  /* report */
  end
  else  /* if we have a name */
  do
   do while (substr(Text,1,2) = Global.CRLF)  /* as long as the text starts with a CRLF */
    Text = substr(Text,3)  /* get rid of it */
   end
   RevText = reverse(Text)  /* reverse the text */
   do while (substr(RevText,1,2) = LFCR)  /* as long as the reverse text starts with a reverse CRLF */
    RevText = substr(RevText,3)  /* get rid of it */
   end
   Text = reverse(RevText)  /* turn the text back again */
   if (Text = '') then  /* if we have no text */
   do
    call AddError 'Entry number '||EntryNo||' contains no text'  /* report */
   end
   RevName = reverse(Name)  /* reverse the name bit */
   parse var RevName . '\' RevPath  /* get the (reverse) path */
   Path = reverse(RevPath)  /* reverse it to get the path */
   Target = AddDir  /* we start here */
   do while (Path >< '')  /* as long as the path continues */
    parse var Path Folder '\' Path  /* get the first part of the path */
    Settings = 'ALWAYSSORT=YES;ICONVIEW=FLOWED,MINI;WORKAREA=YES;'  /* the settings to use for the folder object */
    call syscreateobject 'WPFolder',Folder,Target,Settings,'FAIL'  /* try to create the folder, but fail if it already exists */
    Target = Target||'\'||Folder  /* move up the path for any next bit */
   end
   NewName = ''  /* start with nothing */
   do while (Name >< '')  /* as long as we have bits of name left */
    parse var Name NextBit (Global.CRLF) Name  /* get the bit up to the next CRLF */
    NewName = NewName||NextBit||' '  /* store the next bit */
   end
   NewName = translate(NewName,'()()_____-(..'||"''",'<>[]~?%&*|/;,"'||"`")  /* replace awkward characters */
   OutFile = AddDir||'\'||NewName  /* the name of the output file */
   if (OpenFile(OutFile,'WRITE')) then  /* if we can open the output file */
   do
    call charout OutFile,Text  /* write any text content to the file */
    call stream OutFile,'C','CLOSE'  /* close the file */
    call MessageSettings OutFile,'--------','Change'  /* make it a fresh outgoing mail message file */
    call syssetobjectdata OutFile,'TEMPLATE=YES;'  /* make the file a template */
    do while (substr(Comm,1,2) = Global.CRLF)  /* as long as the keywords start with a CRLF */
     Comm = substr(Comm,3)  /* get rid of it */
    end
    RevComm = reverse(Comm)  /* reverse the keywords */
    do while (substr(RevComm,1,2) = LFCR)  /* as long as the reverse keywords start with a reverse CRLF */
     RevComm = substr(RevComm,3)  /* get rid of it */
    end
    Comm = reverse(RevComm)  /* turn the keywords back again */
    if (Comm >< '') then  /* if we still have keywords */
    do
     call PutKeyPhrases OutFile,Comm  /* add the key phrases */
    end
   end
  end
 end
end

return 1  /* end of PutAddData */

/**********************************************************************/
PutKeyPhrases: procedure expose Global.  /* adds key phrases to an address book entry */
/**********************************************************************/

parse arg File,KeyPhrases  /* get the arguments */

Contents = ''  /* start with nothing */
Count = 0  /* start at 0 */
do while (length(KeyPhrases) > 0)  /* run through the key phrases */
 parse var KeyPhrases Line (Global.CRLF) KeyPhrases  /* get the first line of KeyPhrases */
 Len = reverse(right(d2c(Length(Line)),2,'00'x))  /* set Len */
 Contents = Contents||reverse('FFFD'x)||Len||Line  /* prepare Contents */
 Count = Count + 1  /* up the counter */
end
CodePage = '0000'x  /* set code page */
Count = reverse(right(d2c((Count)),2,'00'x))  /* set Count */
KeyPhrases = reverse('FFDF'x)||CodePage||Count||Contents  /* prepare keywords */
Success = (sysputea(File,'.KEYPHRASES',KeyPhrases) = 0)  /* if we can attach them to the file, we're O.K. */

return Success  /* end of PutKeyPhrases */

/**********************************************************************/
DecodeHeaderLine: procedure expose Global.  /* decodes header strings */
/**********************************************************************/

parse arg String  /* get the parameter */

CodeStart = '=?'  /* define the start of an encoded string */
CodeEnd = '?='  /* define the end of an encoded string */
GotCode = 0  /* we have not seen any encoded stuff yet */
NewString = ''  /* start with nothing */

do while (String >< '')  /* as long as we have a bit of string left */

 parse var String NextBit String  /* get the next bit */

 parse var NextBit FrontBit (CodeStart) CharSet '?' Encoding '?' CodedBit (CodeEnd) BackBit  /* extract the bits we want */

 if (CharSet >< '') then  /* if we found a character set code, we have encoded text */
 do

  select  /* do one of the following */

   when (translate(CharSet) = 'US-ASCII') then  /* if the character set is US-ASCII */
   do
    if (translate(Encoding) = 'Q') then  /* if it uses quoted-printable encoding */
    do
     NewStr = translate(CodedBit,' ','_')  /* decode underscores into spaces */
    end
    else  /* if it uses b64 encoding */
    do
     NewStr = DecodeB64(CodedBit)  /* decode it */
    end
   end

   when (translate(CharSet) = 'ISO-8859-1') then  /* if the character set is ISO-8859-1 */
   do
    if (translate(Encoding) = 'Q') then  /* if it uses quoted-printable encoding */
    do
     NewStr = ''  /* start with nothing */
     do while (CodedBit >< '')  /* as long as we have coded content */
      parse var CodedBit NextChar 2 CodedBit  /* get the first character of what we have left */
      if (NextChar = '=') then  /* if it is a "=" */
      do
       parse var CodedBit NextChar 3 CodedBit  /* get the next two characters */
       NewBit = x2c(NextChar)  /* convert them from hex to character */
      end
      else  /* if it is not a "=" */
      do
       NewBit = translate(NextChar,' ','_')  /* turn an underscore into a space */
      end
      NewStr = NewStr||NewBit  /* add the result to what we have */
     end
    end
    else  /* if it uses b64 encoding */
    do
     NewStr = DecodeB64(CodedBit)  /* decode it */
    end
    NewStr = translate(NewStr,Global.ISOto850,xrange('80'x,'FF'x))  /* convert high bits from ISO 8859-1 to PC850 */
   end

   otherwise  /* if none of the above apply */
   do
    NewStr = '(...)'  /* we have nothing to show until this bit gets added */
   end

  end

  NewStr = FrontBit||NewStr||BackBit  /* stick any front and back bits back on */

  if (\GotCode) then  /* if the previous string contained no code */
  do
   NewStr = ' '||NewStr  /* start with a space */
   GotCode = 1  /* set the code flag */
  end

 end

 else  /* if it is not an encoded string */

 do
  NewStr = ' '||NextBit  /* use it as it is, preceded by a space */
  GotCode = 0  /* reset the code flag */
 end

 NewString = NewString||NewStr  /* add the new bit to the new string */

end

NewString = strip(NewString,'B',' ')  /* remove excess spaces */

return NewString  /* end of DecodeHeaderLine */

/**********************************************************************/
DecodeB64: procedure  /* decodes a B64 string */
/**********************************************************************/

parse arg B64Str  /* get the argument */

B64Chars = xrange('A','Z')||xrange('a','z')||xrange('0','9')||'+/'  /* define the base64 character set */

BitStr = ''  /* start with nothing */

do while (B64Str >< '')  /* as long as we have something left */
 parse var B64Str NextChar 2 B64Str  /* get the next character */
 if (NextChar >< '=') then  /* if it it is not padding */
 do
  CharVal = pos(NextChar,B64Chars) - 1  /* the value represented by the B64 character */
  CharBits = right(x2b(d2x(CharVal)),6,'0')  /* convert the value to 6 bits */
  BitStr = BitStr||CharBits  /* add the result to the bit string */
 end
end

TextStr = ''  /* start with nothing */
do while(length(BitStr) > 7)  /* as long as we have at least a whole byte left */
 parse var BitStr NextChar 9 BitStr  /* get the first 8 bits */
 TextStr = TextStr||x2c(b2x(NextChar))  /* convert to a character and add to the result */
end

return TextStr  /* end of DecodeB64 */

/**********************************************************************/
DispatchMessage: procedure expose Global.  /* dispatches an outgoing message */
/**********************************************************************/

parse arg MessFile  /* collect any arguments */

Success = 0  /* no mail dispatched yet */

if (MessageSettings(MessFile,'-+-*****','Match')) then  /* if the file is an outgoing unprocessed message that's O.K. for processing */
do
 call syssetobjectdata MessFile,'NODELETE=YES;NODRAG=YES;NOMOVE=YES'  /* prevent removal of the file object */

 call MessageContents MessFile  /* get the message file contents */

 DateString = MessageDate(MessFile)  /* as we prepare the file components, we should get back a date string */

 SendFile = MakeSendFile(MessFile)  /* try to get a sendable file */

 call syssetobjectdata MessFile,'NODELETE=NO;NODRAG=NO;NOMOVE=NO'  /* make the message file object movable again */

 if (SendFile >< '') then  /* if we get back a sendable file name */
 do
  parse var Global.RunDispatch FirstBit '%N' LastBit  /* look for the filename macro in the dispatch command line */
  CommLine = FirstBit  /* this is the command to start with */
  if (pos('%N',Global.RunDispatch) > 0) then  /* if the settings line contained a file spec macro */
  do
   CommLine = CommLine||SendFile||LastBit  /* this is the complete command */
  end
  say 'Dispatch configuration option parsed as : '||CommLine  /* report */
  CommLine  /* run the command */
  Success = (RC = 0)  /* if the dispatch command completed without error, all is well */
 end

 if (Success) then  /* if all went well */
 do
  call SetAttachments MessFile,'SEND',DateString  /* rewrite the message EAs and delete the attachments dir */
  call MessageSettings MessFile,'**+*****','Change'  /* new sent mail message setting: Processed */
  call MoveMessage MessFile  /* move the mail file to storage if necessary */
  DateString = subword(SortedDate(DateString),1,2)  /* sort the date string and use the first two words */
  call LogMessage DateString  /* log the message */
  if (Global.Signals) then  /* if a signal is wanted */
  do
   call beep 444,20  /* signal */
   call beep 888,80  /* signal */
  end
 end
 else  /* if something went wrong */
 do
  call AddError 'Could not dispatch "'||MessFile||'"'  /* report */
 end

end

return Success  /* end of DispatchMessage */

/**********************************************************************/
MessageDate: procedure expose Global.  /* prepares message file parts and date string */
/**********************************************************************/

parse arg MessFile  /* get the agrument */

NTime = time('N')  /* get the normal time */
NDate = date('N')  /* get the normal date  */

if (OpenFile(MessFile,'WRITE')) then  /* if we can open the message file for writing */
do
 call charout MessFile,Global.CRLF  /* write a CRLF to the end of the file to set a new date/time stamp */
 call stream MessFile,'C','CLOSE'  /* close the file */
end
else  /* if we cannot open the message file */
do
 call AddError 'Cannot write date stamp to "'||MessFile||'"'  /* just report; this is not a fatal error */
end

Wday = substr(date('W'),1,3)  /* use only the first three letters to comply with RFC 822 */
TimeZone = value('TZ',,'OS2ENVIRONMENT')  /* get the time zone */

DateString = Wday||', '||NDate||' '||NTime||' '||TimeZone  /* prepare the message date string */

MessDate = GetHeaderEntry('DATE:')  /* look for a date line */
if (MessDate = '') then  /* if there is no date */
do
 Global.MessHead = 'Date: '||DateString||Global.CRLF||Global.MessHead  /* add the date to the message header */
end
else  /* if we already have a date in the header */
do
 parse var Global.MessHead FirstPart (MessDate) LastPart  /* get the header parts before and after the original date string */
 Global.MessHead = FirstPart||DateString||LastPart  /* insert the new date into the header */
end

return DateString  /* end of MessageDate */

/**********************************************************************/
EditMessage: procedure expose Global.  /* edits mail file and sets mail message comment */
/**********************************************************************/

parse arg MessFile  /* get the argument */

MessageDir = strip(filespec('D',MessFile)||filespec('P',MessFile),'T','\')  /* the directory the message file is in */
IsAddress = (pos(translate(Global.AddrDir),translate(MessageDir)) > 0)  /* if it is an address folder file, set a flag */
IsBegin = (pos(translate(Global.ConfDir),translate(MessageDir)) > 0)  /* if it is the new message template, set a flag */

if (IsAddress | IsBegin) then  /* if it's an address folder file or the new message template */
do
 EditFile = MessFile  /* we will just edit the file later on */
end
else  /* if it's not an address folder file or the new message template */
do
 call MessageContents MessFile  /* get the message contents */
 Dispatched = (MessageSettings(MessFile,'-*+*****','MATCH'))  /* if the file has been dispatched, set a flag */
 if (Dispatched) then  /* if the file was dispatched */
 do
  DispComm = SetAttachments(MessFile,'LIST')  /* get the dispatch comments, if any */
  Global.MessHead = Global.MessHead||DispComm  /* add the comments to the header */
  HeadText = MakeHeadLines()  /* get the necessary header info from the mail file */
  EditFile = systempfilename(Global.TempDir'\RXML????.SENT')  /* get a unique edit file name */
  EditText = HeadText||Global.MessBody  /* the text to write to the edit file */
 end
 else  /* if the file was not dispatched */
 do
  call MessageSettings MessFile,'*-******','Change'  /* make the file Not O.K. for processing (to prevent accidental sending) */
  if (Global.ASCIIText) then  /* if all messages are to be sent as ASCII text */
  do
   call MessageSettings MessFile,'***+****','Change'  /* set the ASCII text flag */
  end
  AttDir = SetAttachments(MessFile)  /* get the attachments folder, creating it if necessary */
  call sysopenobject AttDir,1,1  /* open the attachments folder */
  call FromChck  /* check for a "From: " line and add closing text if necessary */
  NewHead = HeadChck()  /* check the header entries */
  EditFile = AttDir||'.EDIT'  /* the matching edit file name */
  EditText = NewHead||Global.CRLF||Global.MessBody  /* the text to write to the edit file */
 end
 if (\OpenFile(EditFile,'WRITE')) then  /* if we cannot open the file for writing */
 do
  return 0  /* just quit */
 end
 call charout EditFile,EditText  /* write the edit text to the edit file */
 call stream EditFile,'C','CLOSE'  /* close the edit file */
end

parse var Global.Editor FirstBit '%N' LastBit  /* look for the filename macro in the editor command line */
CommLine = FirstBit  /* this is the command to start with */
if (pos('%N',Global.Editor) > 0) then  /* if the settings line contains a file spec macro */
do
 CommLine = CommLine||EditFile||LastBit  /* this is the complete command */
end
say 'Editor configuration option parsed as : '||CommLine  /* report */
CommLine  /* run the command */

if ((\IsAddress) & (\IsBegin)) then  /* if it's not an address folder file or the new message template */
do
 if (\Dispatched) then  /* if the file was not a dispatched message */
 do
  if ((stream(MessFile,'C','QUERY EXISTS') >< '') & (stream(EditFile,'C','QUERY EXISTS') >< '')) then  /* if both files still exist */
  do
   call MessageContents EditFile  /* get the new edit message contents */
   NewHead = HeadChck()  /* check the header for errors */
   OrgSettings = MessageSettings(MessFile)  /* get the message settings from the original file (they may have changed in the meantime) */
   call sysfiledelete MessFile  /* zap the original message file */

   if (\OpenFile(MessFile)) then  /* if we cannot open a new message file for writing */
   do
    return 0  /* quit with an error */
   end

   call charout MessFile,NewHead||Global.CRLF||Global.MessBody  /* write the new header, a separating empty line, and the body to the message file */
   call stream MessFile,'C','CLOSE'  /* close the message file */
   call MessageSettings MessFile,OrgSettings,'CHANGE'  /* replace the original message status settings */
   NewName = MakeTitle(MessFile)  /* get a new name for the mail file */
   if (pos(Global.Warning,NewName) > 0) then  /* if the header is not O.K. */
   do
    call MessageSettings MessFile,'*-******','CHANGE'  /* set the new message status setting to not O.K. for processing */
   end
   else  /* if the header is O.K. */
   do
    call MessageSettings MessFile,'*+******','CHANGE'  /* set the new message status setting to O.K. for processing */
   end
   call sysfiletree AttDir||'\*.*','Objects.','BO'  /* is there anything in the attachments folder? */
   if (Objects.0 > 0) then  /* if we have attachments */
   do
    NewStore = SetAttachments(MessFile,'CREATE')  /* get a new attachments folder name */
    do Index = 1 to Objects.0  /* take each of the objects */
     if (\sysmoveobject(Objects.Index,NewStore)) then  /* if we cannot move it to the new folder */
     do
      call AddError 'Cannot move "'||Objects.Index||'" to "'||NewStore||'"'  /* report */
     end
    end
   end
   else  /* if there was nothing in the attachments folder */
   do
    call SetAttachments MessFile,'DELETE'  /* get rid of the old attachments folder EA */
   end
   call syssetobjectdata MessFile,'TITLE='||NewName||';'  /* rename the file */
  end
  call sysdestroyobject AttDir  /* try to get rid of the original attachments folder */
 end
 call sysfiledelete EditFile  /* get rid of the edit file */
end

return 1  /* end of EditMessage */

/**********************************************************************/
FromChck: procedure expose Global.  /* checks the sender's address */
/**********************************************************************/

if (GetHeaderEntry('FROM:') = '') then  /* if there is no "From: " line yet */
do
 Global.MessHead = 'From: '||MakeFromLine()||Global.CRLF||Global.MessHead  /* Add a "From: " line containing the default sender */
 CloseFile = Global.ConfDir||'\closing.txt'  /* the name of the closing text file */
 if (OpenFile(CloseFile,'READ')) then  /* if we can open the file, i.e. it exists and is ready */
 do
  Closing = charin(CloseFile,1,chars(CloseFile))  /* copy the contents of the closing phrase file into a buffer */
  call stream CloseFile,'C','CLOSE'  /* close the closing phrase file */
  Global.MessBody = Global.MessBody||Global.CRLF||Global.CRLF||Closing  /* add the contents of the closing phrase file after an empty line */
 end
end

return  /* end of FromChck */

/**********************************************************************/
HeadChck: procedure expose Global.  /* checks header contents */
/**********************************************************************/

AddressKeywords = 'FROM:',  /* allow a from string */
                  'TO:',  /* allow a to string */
                  'CC:',  /* allow a cc string */
                  'BCC:',  /* allow a bcc string */
                  'SENDER:',  /* allow a sender string */
                  'REPLY-TO:',  /* allow a reply to string */
                  'RESENT-FROM:',  /* allow a resent from string */
                  'RESENT-TO:',  /* allow a resent to string */
                  'APPARENTLY-FROM:',  /* allow an apparently-from string */
                  'APPARENTLY-TO:'  /* allow an apparently-to string */

OKKeywords = 'DATE:',  /* allow a date string */
             'SUBJECT:',  /* allow a subject string */
             'COMMENTS:',  /* allow a comments string */
             'IN-REPLY-TO:',  /* allow an in reply to string */
             'REFERENCES:',  /* allow a references string */
             'KEYPHRASES:'  /* allow a keyphrases string */

GlobalHeader = ''  /* start afresh */
NewHeader = ''  /* start afresh */

do while (Global.MessHead >< '')  /* go on while we have bits of unfolded header left */
 parse var Global.MessHead Line (Global.CRLF) Global.MessHead  /* get the next line */
 if (word(Line,1) = Global.Warning) then  /* if the line starts with a marker */
 do
  Line = subword(Line,2)  /* use the remainder */
 end
 FirstWord = word(Line,1)  /* get the first word of the line */
 if (wordpos(translate(Firstword),AddressKeywords) > 0) then  /* if it is in the address keywords list */
 do
  NewLine = FirstWord||' '||FormatAddresses(subword(Line,2),length(FirstWord)+1,1)  /* check and reformat the addresses, indenting to match the keyword, and checking validity */
 end
 else  /* if it is not in the address keywords list */
 do
  if (wordpos(translate(Firstword),OKKeywords) = 0) then  /* if it is not in the O.K. keywords list either */
  do
   if ((substr(Firstword,1,2) >< 'X-') | (substr(Firstword,length(FirstWord),1) >< ':')) then  /* if it is not an X-string followed by a colon, it's a rogue entry */
   do
    Line = Global.Warning||' '||Line  /* so add a marker */
   end
  end
  NewLine = Line  /* use the same line for the folded header */
 end
 NewHeader = NewHeader||NewLine||Global.CRLF  /* add the new line to the new header */
 GlobalHeader = GlobalHeader||Line||Global.CRLF  /* add this line to the unfolded header */
end

Global.MessHead = GlobalHeader  /* copy the unfolded header back to the global variable */

return NewHeader  /* end of HeadChck */

/**********************************************************************/
FormatAddresses: procedure expose Global.  /* formats a string of addres(es) from a mail message header line */
/**********************************************************************/

parse arg Addresses,Indent,Check  /* get the arguments */

First = 1  /* a flag to mark the first address */
NewAddresses = ''  /* start with no new addresses */
Check = (Check = 1)  /* if the check arg is 1, we need to check the addresses */

do while (Addresses >< '')  /* as long as we have addresses */

 if (word(Addresses,1) = Global.Warning) then  /* if the next address starts with a marker */
 do
  Addresses = subword(Addresses,2)  /* use the rest */
 end

 Addresses = strip(Addresses,'B',' ')  /* remove excess blanks */

 if (Addresses >< '') then  /* if we still have something */
 do

  if (substr(Addresses,1,1) = '"') then  /* if it starts with a quotation mark, it must be a comment preceding the actual address */
  do
   parse var Addresses '"' Comment '"' Address ',' Addresses  /* get the bits we need */
   Address = strip(Address,'B',' ')  /* get rid of excess spaces */
   OrgAddress = '"'||Comment||'" '||Address  /* this was the original address */
   parse var Address FirstBit '@' Remainder  /* look for the bits around a "@" */
   Remainder = strip(Remainder,'B',' ')  /* get rid of excess spaces */
   Comment = strip(Comment,'B',"'")  /* get rid of any single quotes around the comment */
  end
  else  /* if it does not start with a quotation mark */
  do
   parse var Addresses Address ',' Addresses  /* get the whole lot up to the next comma */
   Address = strip(Address,'B',' ')  /* get rid of excess spaces */
   OrgAddress = Address  /* this was the original address */
   parse var Address FirstBit '@' Remainder  /* look for the bits around a "@" */
   if (words(FirstBit) > 1) then  /* if the first bit contains more than one word */
   do
    Comment = subword(FirstBit,1,words(FirstBit)-1)  /* all words but the last form an unquoted comment */
    Comment = translate(Comment,"'",'"')  /* change any double quotation marks into single ones */
    Comment = strip(Comment,'B',"'")  /* get rid of any single quotes around the comment */
    FirstBit = word(FirstBit,words(FirstBit))  /* use the last word for the address */
   end
   else  /* if we have only one word */
   do
    Comment = ''  /* we have no comment */
   end
  end

  parse var Remainder LastBit EndComment  /* use the first word of the last bit for the real address; any remainder must be an end comment */
  EndComment = strip(EndComment,'T',' ')  /* remove trailing blanks */
  FirstBit = strip(FirstBit,'L',' ')  /* remove any leading blanks from the first bit */
  LastBit = strip(LastBit,'T',' ')  /* remove any trailing blanks from the last bit */

  Address = CheckAddressComponents(FirstBit,LastBit)  /* see if the address components are O.K., i.e. they make up a new address */
  if ((Address >< '') & (EndComment >< '')) then  /* if we still have an address, and an end comment */
  do
   if ((substr(EndComment,1,1) >< '(') | (substr(EndComment,length(EndComment)) >< ')')) then  /* if the end comment is not between brackets */
   do
    Address = ''  /* the address is bad, so zap it */
   end
  end

  if (Address = '') then  /* if the address was zapped (because it was bad) */
  do
   Address = OrgAddress  /* use the original address */
   if (Check) then  /* if we are checking the address */
   do
    Address = Global.Warning||' '||Address  /* add a marker */
   end
  end
  else  /* if all is well */
  do
   if (pos(Address,NewAddresses) = 0) then  /* if it is one we haven't got yet */
   do
    if (EndComment >< '') then  /* if we have an end comment */
    do
     if (Comment >< '') then  /* if we have a front comment */
     do
      Comment = Comment||' - '  /* stick on a separator */
     end
     EndComment = strip(EndComment,'L','(')  /* get rid of the leading bracket */
     EndComment = strip(EndComment,'T',')')  /* get rid of the trailing bracket */
     EndComment = strip(EndComment,'B','"')  /* remove any double quotation marks */
     Comment = Comment||EndComment  /* append the end comment to the front comment, if any */
    end
    if ((Comment >< '') & ('<'||Comment||'>' >< Address)) then  /* if we have a comment and it's not the same as the address */
    do
     Address = '"'||Comment||'" '||Address  /* stick it on the front, between double quotes */
    end
   end
   else  /* if we've already got this one */
   do
    Address = ''  /* we have no Address */
   end
  end

  if (First) then  /* if this was the first address */
  do
   First = 0  /* the next is no longer the first */
  end
  else  /* if it wasn't the first address */
  do
   if (Address >< '') then  /* if we have something  */
   do
    Address = ','||Global.CRLF||copies(' ',Indent)||Address  /* start an (indented) new line after a comma */
   end
  end

  NewAddresses = NewAddresses||Address  /* add it to the result */

 end

end

return NewAddresses  /* end of FormatAddresses */

/**********************************************************************/
CheckAddressComponents: procedure expose Global.  /* check an address's components */
/**********************************************************************/

parse arg FirstBit,LastBit  /* get the arguments */

if ((words(FirstBit) >< 1) | (words(LastBit) >< 1)) then  /* if the first bit does not contain exactly one word, and the last bit likewise */
do
 return ''  /* return with nothing */
end

GotStartBracket = (substr(FirstBit,1,1) = '<')  /* have we got a leading angle bracket on the first bit? */
GotEndBracket = (substr(LastBit,length(LastBit)) = '>')  /* have we got a trailing angle bracket on the last bit? */

if (GotStartBracket >< GotEndBracket) then  /* if the start of the first bit and the end of the last bit do not match */
do
 return ''  /* return with nothing */
end

if (GotStartBracket) then  /* if the first bit has a leading angle bracket */
do
 FirstBit = strip(FirstBit,'L','<')  /* get rid of the leading angle bracket on the first bit */
 if (words(FirstBit) = 0) then  /* if the first bit no longer contains a word */
 do
  return ''  /* return with nothing */
 end
end

if (GotEndBracket) then  /* if we have a trailing angle bracket on the last bit */
do
 LastBit = strip(LastBit,'T','>')  /* get rid of the trailing angle bracket */
 if (words(LastBit) = 0) then  /* if the last bit no longer contains a word */
 do
  return ''  /* return with nothing */
 end
end

BadRange = '()[]@,;:\" '||,  /* we can't have these in an e-mail address */
           xrange('00'x,'1F'x)||,  /* nor these */
           xrange('7F'x,'FF'x)  /* nor these */

if (verify(FirstBit,'>'||BadRange,'M',1) > 0) then  /* if the first bit contains illegal characters */
do
 return ''  /* return with nothing */
end

if (verify(LastBit,'<'||BadRange,'M',1) > 0) then  /* if the last bit contains illegal characters */
do
 return ''  /* return with nothing */
end

Address = '<'||FirstBit||'@'||LastBit||'>'  /* add angle brackets around the complete address */

return Address  /* end of CheckAddressComponents */

/**********************************************************************/
GetFileEntry: procedure expose Global.  /* looks for key word in files and returns last line with matching content */
/**********************************************************************/

parse arg FileName,SeekStr  /* the usual stuff */

SeekStr = ' '||SeekStr||';'  /* add a space and semi-colon */

if (sysfilesearch(SeekStr,FileName,'Found.') >< 0) then  /* if we cannot search the file for the seek string */
do
 call AddError 'Cannot search for "'||SeekStr||'" in "'||FileName||'"'  /* report */
 return ''  /* and quit with nothing */
end

OutLine = ''  /* start with no result */

do Index = 1 to Found.0  /* look through the lines we found */
 if (substr(Found.Index,1,1) >< '#') then  /* if the line we found is not a comment */
 do
  parse var Found.Index OutLine '=' .  /* the bit before the '=' is what we're looking for */
 end
end

if (OutLine >< '') then  /* if we have a result */
do
 OutLine = translate(OutLine,d2c(32),d2c(9))  /* convert TABs into spaces */
 OutLine = strip(OutLine)  /* get rid of any excess spaces */
end

return OutLine  /* end of GetFileEntry */

/**********************************************************************/
GetHeaderEntry: procedure expose Global.  /* returns entry for matching key word in message header */
/**********************************************************************/

parse upper arg SeekStr,More  /* get the seek string in upper case, and any "MORE" parameter */

if (More = '') then  /* if we have no "More" parameter */
do
 More = 0  /* use no more */
end

OutLine = ''  /* start with nothing */

Header = Global.MessHead  /* copy the header contents */

do while ((Header >< '') & ((OutLine = '') | (More)))  /* go on until we run out of header content or find what we want */
 parse var Header HeaderLine (Global.CRLF) Header  /* get the relevant bits */
 if (SeekStr = translate(word(HeaderLine,1))) then  /* if the uppercased first word is the one we're looking for */
 do
  Entry = subword(HeaderLine,2)  /* the bit we want */
  if (Entry = '') then  /* if there is nothing in it */
  do
   Entry = d2c(9)  /* use a single tab */
  end
  if (OutLine >< '') then  /* if we already have something */
  do
   Entry = Global.CRLF||Entry  /* add a new line */
  end
  OutLine = OutLine||Entry  /* add the new bit to the output text */
 end
end

return OutLine  /* end of GetHeaderEntry */

/**********************************************************************/
HelpText: procedure expose Global.  /* displays a help message */
/**********************************************************************/

say 'Usage : RexxMail /switch [filespec]'  /* send a line to StdOut */
say 'Switch options (filespec may contain wildcards) :'  /* send a line to StdOut */
say '  /help                       - display this help message'  /* send a line to StdOut */
say '  /info                       - view info files'  /* send a line to StdOut */
say '  /open                       - open mail message folders'  /* send a line to StdOut */
say '  /edit filespec              - edit outgoing message file'  /* send a line to StdOut */
say '  /dispatch                   - dispatch messages from out folder if ready'  /* send a line to StdOut */
say '  /dispatch filespec          - dispatch outgoing message if ready'  /* send a line to StdOut */
say '  /register filespec          - register incoming message'  /* send a line to StdOut */
say '  /view filespec              - view incoming message'  /* send a line to StdOut */
say '  /raw filespec               - view raw incoming message'  /* send a line to StdOut */
say '  /reply filespec             - create reply message'  /* send a line to StdOut */
say '  /replytoall filespec        - create reply message to all addresses'  /* send a line to StdOut */
say '  /replytorecipients filespec - create reply message to "From:" address'  /* send a line to StdOut */
say '  /replytosender filespec     - create reply message to "To:" address'  /* send a line to StdOut */
say '  /address                    - opens addresses folder'  /* send a line to StdOut */
say '  /address filespec           - add address to addresses folder'  /* send a line to StdOut */
say '  /export                     - export address text data'  /* send a line to StdOut */
say '  /import                     - import address text data'  /* send a line to StdOut */
say '  /setdispatched filespec     - set message attributes to "dispatched"'  /* send a line to StdOut */
say '  /setnew filespec            - set message attributes to "new"'  /* send a line to StdOut */
say '  /setreceived filespec       - set message attributes to "received"'  /* send a line to StdOut */
say '  /setviewed filespec         - set message attributes to "viewed"'  /* send a line to StdOut */

return 1  /* end of HelpText */

/**********************************************************************/
LogMessage: procedure expose Global.  /* log a dispatched or registered message */
/**********************************************************************/

if (Global.MailLogs) then  /* if we are to log messages */
do
 MailLog = Global.ConfDir||'\maillogs.txt'  /* the name of the mail log file */
 if (OpenFile(MailLog,'WRITE')) then  /* if we can open the file for writing */
 do
  parse arg DTStamp  /* get the argument */
  if (DTStamp = '') then  /* if there is no date/time stamp argument, this must be an incoming message */
  do
   DTStamp = substr(date('S'),1,4)||'-'||,  /* start the date & time stamp with the year in 4 digits */
             substr(date('S'),5,2)||'-'||,  /* followed by the month in 2 digits */
             substr(date('S'),7,2)||' '||,  /* followed by the day of the month in 2 digits */
             time('N')||' <==  '  /* followed by the normal time and an "incoming" marker */
  end
  else  /* if we have a date/time stamp argument, this must be an outgoing message */
  do
   DTStamp = DTStamp||'  ==> '  /* add an "outgoing" marker */
  end

  LookFor = 'From:',  /* look for this entry in the header, */
            'Apparently-From:',  /* then this one */
            'Sender:',  /* then this one */
            'From:'  /* then this one */
  Sender = GetLogEntry('sender',LookFor)  /* get the sender string */

  LookFor = 'To:',  /* look for this entry in the header, */
            'Apparently-To:'  /* then this one */
  Recipients = GetLogEntry('recipients',LookFor)  /* get the recipient(s) string */

  LookFor = 'Subject:'  /* look for this entry in the header */
  Subject = GetLogEntry('subject',LookFor)  /* get the subject string */

  call lineout MailLog,DTStamp||Sender||' -- '||Recipients||' -- '||Subject  /* write the log line */
  call stream MailLog,'C','CLOSE'  /* close the log file */
 end
end

return 1  /* end of LogMessage */

/**********************************************************************/
GetLogEntry: procedure expose Global.  /* gets the sender for the log entry */
/**********************************************************************/

parse arg EntryName,LookFor  /* get the argument */

Entry = ''  /* nothing yet */

do until ((Entry >< '') | (LookFor = ''))  /* go on until we find something or run out of options */
 parse var LookFor NextOne LookFor  /* get the next option */
 Entry = GetHeaderEntry(NextOne)  /* look for the entry in the message header */
end

if (Entry >< '') then  /* if we have something */
do
 Entry = NextOne||' '||Entry  /* combine the two bits */
end
else  /* if we have nothing */
do
 Entry = '(no '||EntryName||')'  /* use this */
end

return Entry  /* end of GetLogEntry */

/**********************************************************************/
MailWarning: procedure expose Global.  /* shows that mail has arrived, or deletes its own program object */
/**********************************************************************/

Success = 0  /* no success yet */

ProgID = '<RexxMail_Warning_Object>'  /* the "Mail!" warning program object ID */

parse upper arg Param  /* get the parameter */

if (Param = 'DELETE') then  /* if this is the parameter */
do
 if (sysdestroyobject(ProgID)) then  /* if we can remove the warning object */
 do
  Success = 1  /* all is well */
 end
end
else  /* if there is no such parameter */
do
 if (Global.WarnIcon) then  /* if we want a warning icon */
 do
  GotMain = 0  /* we have not spotted the main mail folder yet */
  call sysqueryswitchlist 'Stem.'  /* store the window list contents in Stem. */
  do Index = 1 to Stem.0  /* take each entry */
   if (translate(Stem.Index) = filespec('N',Global.MainDir)) then  /* if it is identical to the main mail folder */
   do
    GotMain = 1  /* the mail folder must be open on the desktop */
   end
  end
  if (\GotMain) then  /* if the main mail folder is not open on the desktop */
  do
   Settings = 'EXENAME='||Global.ProgDir||Global.ProgName||';'||,  /* opening the object will call RexxMail */
              'PARAMETERS=/OPEN;MINIMIZED=YES;'||,  /* the program will run minimized, with the "/Open" switch */
              'OBJECTID='||ProgID||';'||,  /* use the object ID defined earlier */
              'ICONFILE='||Global.IconDir||'\mailwarn.ico'  /* attach the MailWarn icon file */
   Success = (syscreateobject('WPProgram','Mail!','<WP_DESKTOP>',Settings))  /* all is well if we can create the program object */
  end
 end
end

return Success  /* end of MailWarning */

/**********************************************************************/
MakeAddressEntry: procedure expose Global.  /* extracts address from mail file and adds a template to address book folder */
/**********************************************************************/

parse arg MessFile  /* get the argument */

call MessageContents MessFile  /* get the message contents */

if (MessageSettings(MessFile,'+*******','MATCH')) then  /* if it is an incoming message */
do
 Address = GetHeaderEntry('FROM:')  /* use this address */
end
else  /* if it is not an incoming message */
do
 if (MessageSettings(MessFile,'-*******','MATCH')) then  /* if it is an outgoing message */
 do
  Address = GetHeaderEntry('TO:')  /* use this address */
 end
 else  /* if it is not an outgoing message either */
 do
  call AddError 'Not a valid mail message'  /* report */
  return 0  /* and quit */
 end
end

Address = DecodeHeaderLine(Address)  /* decode the address if necessary */
Address = FormatAddresses(Address,4,0)  /* reformat the address, indent to match "To: ", no check */

if ((pos(Address,d2c(9)||' ') > 0) | (pos(Global.Warning,Address) > 0)) then  /* if there is no address, or if it contains a marker */
do
 call AddError 'No valid address found'  /* report */
 return 0  /* and quit */
end

Title = translate(Address," ()",'"<>')  /* change dangerous (command line!) characters for the object title */
Title = strip(Title,'B',' ')  /* get rid of excess spaces */
Title = '! '||Title||d2c(255)  /* stick on a marker, and a hard space to prevent file filter problems */

OutFile = Global.AddrDir||'\'||Title  /* the file to write to */

if (stream(OutFile,'C','QUERY EXISTS') >< '') then  /* if such a file already exists in the address directory */
do
 call AddError 'Address template already exists'  /* report */
 return 0  /* and quit */
end

OutText = 'To: '||Address||Global.CRLF||'Subject: '  /* line up the output text */

if (\OpenFile(OutFile)) then  /* if we cannot open the file */
do
 return 0  /* and quit */
end

call charout OutFile,OutText  /* write the output text to the file */
call stream OutFile,'C','CLOSE'  /* close the file */

if (\MessageSettings(OutFile,'--------','Change')) then  /* if we can not make it a fresh outgoing mail message file */
do
 call AddError 'Cannot set address template message attributes'  /* report */
 return 0  /* and quit */
end

if (\syssetobjectdata(OutFile,'TEMPLATE=YES;')) then  /* if we cannot make the file a template */
do
 call AddError 'Cannot set address template attribute'  /* report */
 return 0  /* and quit */
end

return 1  /* end of MakeAddressEntry */

/**********************************************************************/
MakeFromLine: procedure expose Global.  /* make a "From:" line */
/**********************************************************************/

if (Global.Address >< '') then  /* if we have a default sender's address */
do
 FromLine = strip(Global.Address,'L','<')  /* get rid of any leading angle bracket */
 FromLine = strip(FromLine,'T','>')  /* get rid of any trailing angle bracket */
 FromLine = '<'||FromLine||'>'  /* add angle brackets */
 if (Global.Name >< '') then  /* if we have a default sender's name */
 do
  FromName = strip(Global.Name,'B','"')  /* get rid of any double quotes */
  FromName = strip(FromName,'B',"'")  /* get rid of any single quotes */
  FromLine = '"'||FromName||'" '||FromLine  /* add double quotes and put the name in front of the address */
 end
end
else  /* if we do not have a default sender's address */
do
 FromLine = ''  /* start with nothing */
end

return FromLine  /* end of MakeFromLine */

/**********************************************************************/
MakeHeadLines: procedure expose Global.  /* gets header info from original message file and creates header text block */
/**********************************************************************/

parse arg KeywordList  /* get the argument */

Header = ''  /* start with nothing */
Header. = ''  /* start with nothing */
Counter = 0  /* start at 0 */
MaxLength = 0  /* start at 0 */

Reformats = 'FROM',  /* reformat addresses */
            'TO',  /* reformat addresses */
            'REPLY-TO',  /* reformat addresses */
            'CC',  /* reformat addresses */
            'BCC',  /* reformat addresses */
            'APPARENTLY-FROM',  /* reformat addresses */
            'SENDER',  /* reformat addresses */
            'RESENT-TO',  /* reformat addresses */
            'RESENT',  /* reformat addresses */
            'FROM',  /* reformat addresses */
            'ATTACHMENTS'  /* reformat addresses */

Separator = copies('_',32)||Global.CRLF||Global.CRLF  /* the separator between the reader header and the message body */

if (KeywordList >< '') then  /* if we have a list of keywords */
do while(KeywordList >< '')  /* while we have something left in this variable */
 parse var KeywordList Nextword KeywordList  /* get the next word from the list of keywords */
 if (substr(NextWord,1,1) = '*') then  /* if it starts with "*" */
 do
  NextWord = substr(NextWord,2)  /* use the rest */
  More = 1  /* we want more than the first occurrence */
 end
 else  /* if the word does not start with "*" */
 do
  More = 0  /* we don't want more */
 end
 NextEntry = GetHeaderEntry(NextWord||':',More)  /* look for this entry in the message header */
 call ProcessEntry  /* process the entry we have */
end
else  /* if we do not have a list of keywords, we will use the whole header */
do
 MessHeader = Global.MessHead  /* copy the header contents */
 do while(MessHeader >< '')  /* while we have something left in the header text */
  parse var MessHeader NextWord NextEntry (Global.CRLF) MessHeader  /* get the bits we want */
  if (substr(nextWord,length(NextWord)) = ':') then  /* if this bit ends with a colon */
  do
   NextWord = strip(NextWord,'T',':')  /* get rid of it */
   call ProcessEntry  /* process the entry we have */
  end
 end
end

do Index = 1 to Counter  /* get each one of the bits we have */
 HeadWord = subword(Header.Index,1,1)  /* the first word */
 HeadLine = subword(Header.Index,2)  /* the second word and the rest */
 OutLine = left(HeadWord,MaxLength,' ')||' : '  /* start with the first word, padded to the maximum length, followed by space, colon and space */
 do while (HeadLine >< '')  /* as long as we have something left */
  parse var HeadLine NextLine (Global.CRLF) HeadLine  /* get the next line */
  OutLine = OutLine||NextLine  /* add it to the bit we already have */
  if (HeadLine >< '') then  /* if we have more */
  do
   OutLine = OutLine||Global.CRLF||copies(' ',MaxLength+3)  /* add a new line and indent */
  end
 end
 Header = Header||OutLine||Global.CRLF  /* and add the result to the header */
end

if (Header >< '') then  /* if we have a header */
do
 Header = Header||Separator  /* add the separator */
end

return Header  /* end of MakeHeadLines */

/**********************************************************************/
ProcessEntry:  /* processes an header entry for inclusion in a reader header */
/**********************************************************************/

if (NextEntry >< '') then  /* if we found something */
do
 if (wordpos(translate(NextWord),'DATE SENT RECEIVED') > 0) then  /* if it is one of these */
 do
  if (translate(NextWord) = 'RECEIVED') then  /* if it is this one */
  do
   parse var NextEntry RouteInfo ';' NextEntry  /* try to get just the received date */
  end
  NextEntry = SortedDate(NextEntry)  /* rewrite the date if necessary */
 end
 if (wordpos(translate(NextWord),Reformats) > 0) then  /* if this is one we want to reformat */
 do
  NextEntry = FormatAddresses(NextEntry,0,1)  /* get a formatted string of addresses, no indent, no checking */
 end
 if (NextEntry >< '') then  /* if we still have something */
 do
  Counter = Counter + 1  /* up the counter */
  Header.Counter = NextWord||' '||DecodeHeaderLine(NextEntry)  /*  add it to the reader header contents after decoding it */
  ThisLength = Length(NextWord)  /* get the length of the key word */
  if (ThisLength > MaxLength) then  /* if it is longer than the longest so far */
  do
   MaxLength = ThisLength  /* use the new length */
  end
 end
end

return  /* end of ProcessEntry */

/**********************************************************************/
MakeReplyMessage: procedure expose Global.  /* generate a reply message */
/**********************************************************************/

parse arg MessFile,Switch  /* get the arguments */

AttDir = UnpackMessage(MessFile)  /* unpack the mail file and get the attachments directory name (and the message contents) */
ReadFile = AttDir||'.READ'  /* the reader file name */

MessFileDir = filespec('D',MessFile)||filespec('P',MessFile)  /* get the path to the mail message */

if (pos(translate(Global.MainDir),translate(MessFileDir)) > 0) then  /* if it is anywhere in the RexxMail (sub)folders */
do
 ReplyDir = Global.OutDir||'\'  /* create the reply in the 'Out' folder */
end
else  /* if the original message is not in any of the RexxMail folders */
do
 ReplyDir = MessFileDir  /* create the reply file in the same folder as the original */
end

if (\OpenFile(ReadFile,'READ')) then  /* if we cannot open the message read file for reading */
do
 return 0  /* and return with an error */
end

ReadText = charin(ReadFile,1,chars(ReadFile))  /* copy the contents of the file into a buffer */
call stream ReadFile,'C','CLOSE'  /* close the read file */

parse var ReadText . (Global.EmptyLine) BodyText  /* get the message body text */

ReplyFile = systempfilename(ReplyDir||'RXML????.REPLY')  /* create a unique temporary file name */

if (\OpenFile(ReplyFile,'WRITE')) then  /* if we cannot open the reply file for writing */
do
 return 0  /* and return with an error */
end

Sender = GetHeaderEntry('FROM:')  /* get the sender; we need it anyway to quote in the message */
if (Sender = '') then  /* if we have nothing */
do
 Sender = GetHeaderEntry('SENDER:')  /* look for the real sender */
 if (Sender = '') then  /* if we have nothing */
 do
  Sender = GetHeaderEntry('APPARENTLY-FROM:')  /* look for the apparent sender */
  if (Sender = '') then  /* if we have nothing */
  do
   Sender = GetHeaderEntry('FROM')  /* look for the envelope sender */
   if (Sender >< '') then  /* if we have something this time */
   do
    Sender = word(Sender,1)  /* use just the first word */
   end
  end
 end
end

select  /* do one of the following */

 when (Switch = 'REPLYTOALL') then  /* if we are replying to everybody */
 do
  SendTo = Sender  /* start with the sender's address */
  ReplyToAddress = GetHeaderEntry('REPLY-TO:')  /* look for "Reply-To:" addresses */
  if (ReplyToAddress >< '') then  /* if we have something */
  do
   SendTo = SendTo||', '||ReplyToAddress  /* add it to the recipients */
  end
  ToAddress = GetHeaderEntry('TO:')  /* look for "To:" addresses */
  if (ToAddress >< '') then  /* if we have something */
  do
   SendTo = SendTo||', '||ToAddress  /* add it to the recipient list */
  end
  CcAddress = GetHeaderEntry('CC:')  /* look for "Cc:" addresses */
  if (CcAddress >< '') then  /* if we have something */
  do
   SendTo = SendTo||', '||CcAddress  /* add it to the recipient list */
  end
 end

 when (Switch = 'REPLYTOSENDER') then  /* if we are replying to the "From:" address */
 do
  SendTo = Sender  /* use the sender's address */
 end

 when (Switch = 'REPLYTORECIPIENTS') then  /* if we are replying to the "To:" addresses */
 do
  SendTo = GetHeaderEntry('TO:')  /* get the "To:" addresses */
  if (SendTo = '') then  /* if there is no address */
  do
   SendTo = GetHeaderEntry('APPARENTLY-TO:')  /* look for an "Apparently-To:" address */
  end
 end

 otherwise  /* if we have none of the above special cases */
 do
  SendTo = GetHeaderEntry('REPLY-TO:')  /* try to find a "Reply-To:" address */
  if (SendTo = '') then  /* if there is none */
  do
   SendTo = Sender  /* use the sender address */
  end
 end

end

if (SendTo = '') then  /* if we have no address */
do
 call AddError 'No address found'  /* report */
 return 0  /* and return with an error */
end

Sendto = DecodeHeaderLine(SendTo)  /* decode any MIME stuff */
OutText = 'To: '||SendTo  /* add the address line */

MessageID = GetHeaderEntry('Message-ID:')  /* try to get a message ID */
if (MessageID <> '') then  /* if there is a message ID */
do
 OutText = OutText||Global.CRLF||'In-Reply-To: '||MessageID  /* add the reply info line */
end

Subject = GetHeaderEntry('SUBJECT:')  /* the original subject */
if (Subject = '') then  /* if we have no subject */
do
 Subject = '(no subject)'  /* substitute this */
end
else  /* if we have a subject */
do
 Subject = DecodeHeaderLine(Subject)  /* decode any MIME stuff */
 if (translate(word(Subject,1)) >< 'RE:') then  /* if it does not start with "Re:" */
 do
  Subject = 'Re: '||Subject  /* add it */
 end
end
OutText = OutText||Global.CRLF||'Subject: '||Subject  /* add this subject line */

if (Sender >< '') then  /* if we have a sender */
do
 Sender = FormatAddresses(sender,0,1)  /* reformat the address, no indent, check */
 Sender = DecodeHeaderLine(Sender)  /* decode any MIME stuff */
end
else  /* if we still have nothing */
do
 Sender = 'an unknown sender'  /* use this */
end

MessDate = GetHeaderEntry('DATE:')  /* try to get a sent date */
if (MessDate >< '') then  /* if we have a date */
do
 LineStart = 'In a message dated '||SortedDate(MessDate)  /* rewrite the date and stick some text in front */
end
else  /* if we have no date */
do
 LineStart = 'In a previous message'  /* use this */
end

OutText = OutText||Global.CRLF||Global.CRLF||,  /* add an empty line to the output text */
          LineStart||','||Global.CRLF||,  /* add this info line to the message */
          Sender||' wrote:'||Global.CRLF  /* add this info line to the message */

do while (BodyText >< '')  /* while we have something in the body text */
 parse var BodyText OutLine (Global.CRLF) BodyText  /* take each line */
 OutText = OutText||Global.CRLF||'> '||OutLine  /* and add it to the output text, preceded by a quote sign */
end

call charout ReplyFile,OutText  /* write the output text to the reply message file */
call stream ReplyFile,'C','CLOSE'  /* close the reply file */

call MessageSettings ReplyFile,'--------','CHANGE'  /* make it an outgoing mail message file */

call MessageContents ReplyFile  /* get the contents of the new file */

NewName = MakeTitle(ReplyFile)  /* get the new name */

if (\syssetobjectdata(ReplyFile,'TITLE='||NewName)) then  /* if we cannot rename the file */
do
 call AddError 'Cannot rewrite title of "'||ReplyFile||'"'  /* report; not fatal */
end

if (\sysdestroyobject(AttDir)) then  /* if we cannot zap the attachments folder */
do
 call AddError 'Cannot delete "'||AttDir||'"'  /* report */
end

if (stream(ReadFile,'C','QUERY EXISTS') >< '') then  /* if the read file still exists */
do
 if (sysfiledelete(ReadFile) > 0) then  /* if we cannot get rid of the read file */
 do
  call AddError 'Cannot delete '||ReadFile  /* report */
 end
end

return 1  /* exit */


/**********************************************************************/
MakeSendFile: procedure expose Global.  /* prepares the actual file to send */
/**********************************************************************/

parse arg MessFile  /* get the argument */

ASCIIMessage = MessageSettings(MessFile,'-**+****','MATCH')  /* if we find an ASCII EA, this message should be sent as ASCII */

AttDir = SetAttachments(MessFile,'GETDIR')  /* extract the attachments directory name if there is one */

SendFile = systempfilename(Global.TempDir||'\RXML????.SEND')  /* prepare a unique file name */

MessHead = Global.MessHead  /* copy the header contents */
MessBody = Global.MessBody  /* copy the body contents */

MessHead = translate(MessHead,Global.ESCtoASC,xrange('00'x,'1F'x))  /* convert header escape codes to ASCII */
MessBody = translate(MessBody,Global.ESCtoASC,xrange('00'x,'1F'x))  /* convert body text escape codes to ASCII */

if (ASCIIMessage) then  /* if we want ASCII text */
do
 MessHead = translate(MessHead,Global.850toASC,xrange('80'x,'FF'x))  /* convert the header to ASCII*/
 MessBody = translate(MessBody,Global.850toASC,xrange('80'x,'FF'x))  /* convert the body text to ASCII */
end
else  /* if we want quoted-printable ISO 8859-1 text */
do
 MessHead = translate(MessHead,Global.850toISO,xrange('80'x,'FF'x))  /* convert the header to ISO 8859-1 */
 MessBody = translate(MessBody,Global.850toISO,xrange('80'x,'FF'x))  /* convert the body text to ISO 8859-1 */
end

NewHead = ''  /* start with nothing */

do until (MessHead = '')  /* go on until we run out of header */

 MIMEPrevWord = 0  /* there is no previous word, so it cannot have been a MIME word */

 parse var MessHead NextLine (Global.CRLF) MessHead  /* get a line from the header */
 parse var NextLine NewLine NextLine  /* start the new line with the first word, which is a keyword */

 do while (NextLine >< '')  /* go on until we run out of line */

  parse var NextLine NextWord NextLine  /* get the next word in the line */

  if (verify(NextWord,xrange(x2c(00),x2c(7F)),'NOMATCH') > 0) then  /* if the current word contains any high-bit characters */

  do

   NewWord = ''  /* start with nothing */
   ISOStart = '=?iso-8859-1?Q?'  /* define the start of an ISO string */
   ISOEnd = '?='  /* define the end of an ISO string */

   if (substr(NextWord,1,1) = '"') then  /* if the word starts with a quotation mark */
   do
    NextWord = substr(NextWord,2)  /* cut it off */
    ISOStart = '"'||ISOStart  /* add the quotation mark to the start of the ISO start code */
   end

   if (substr(NextWord,length(NextWord),1) = '"') then  /* if the word ends with a quotation mark */
   do
    NextWord = substr(NextWord,1,length(NextWord)-1)  /* cut it off */
    ISOEnd = ISOEnd||'"'  /* add the quotation mark to the end of the ISO end code */
   end

   do Index = 1 to length(NextWord)  /* run through the message text */
    NextChar = substr(NextWord,Index,1)  /* take each character */
    if ((c2d(NextChar) > 126) | (pos(NextChar,'_=') > 0)) then  /* if it's not in the OK range, or if we are in a MIME word and it is an underscore or equal sign */
    do
     NewWord = NewWord||'='||c2x(NextChar)  /* substitute an equals sign followed by the character's hex code */
    end
    else  /* if the character is in the OK range */
    do
     NewWord = NewWord||NextChar  /* simply add it to the output buffer */
    end
   end

   if (MIMEPrevWord) then  /* if the previous word was also a MIME word */
   do
    NewWord = '_'||NewWord  /* stick on an underscore */
   end
   else  /* if the previous word was not a MIME word */
   do
    NewWord = ' '||ISOStart||NewWord  /* stick on a space and an ISO start string */
    MIMEPrevWord = 1  /* set the MIME flag */
   end

  end

  else  /* if the word does not contain any high-bit characters */

  do
   if (MIMEPrevWord) then  /* if the previous word was a MIME word */
   do
    NewWord = ISOEnd||' '||NextWord  /* stick on an ISO end string and a space */
    MIMEPrevWord = 0  /* reset the MIME flag */
   end
   else  /* if the previous word was not a MIME word */
   do
    NewWord = ' '||NextWord  /* just stick on a space */
   end
  end

  NewLine = NewLine||NewWord  /* add it to the rest of the line */

 end

 NewHead = NewHead||NewLine||Global.CRLF  /* add the new line to the header */

end

NewHead = NewHead||'X-mailer: '||Global.Build||Global.CRLF  /* "X-mailer" line for the header */

GotAttach = 0  /* no attachments yet */
if (AttDir >< '') then  /* if there is an attachment storage directory */
do
 call sysfiletree AttDir||'\*.*','Attach.','FOS'  /* look for attachment files in the attachments directory and any subfolders */
 if (Attach.0 > 0) then  /* if we found any */
 do
  GotAttach = 1  /* we've got MIME attachments */
  MIMETypeFile = Global.ConfDir||'\mimetype.txt'  /* the list of MIME types */
  if (Global.RunAttachOut >< '') then  /* if we are to run a command in the attachments folder */
  do
   CurDir = directory()  /* store the current directory name */
   if (directory(AttDir) >< '') then  /* if we can change to the attachments directory */
   do
    parse var Global.RunAttachOut FirstBit '%N' LastBit  /* look for the filename macro in the RunAttachOut command line */
    CommLine = FirstBit  /* this is the command to start with */
    if (pos('%N',Global.RunAttachOut) > 0) then  /* if the settings line contains a file spec macro */
    do
     CommLine = CommLine||date('D')||time('S')||LastBit  /* this is the complete command */
    end
    say 'RunAttachOut configuration option parsed as : '||CommLine  /* report */
    CommLine  /* run the command */
    call sysfiletree AttDir||'\*.*','Attach.','FOS'  /* look again for remaining attachment files in the attachments directory and any subdirectories */
    if (directory(CurDir) = '') then  /* if we cannot change back to the original directory */
    do
     call AddError 'Cannot change directory to: "'||CurDir||'"'  /* report */
     return ''  /* and quit */
    end
   end
   else  /* if we cannot change to the attachments directory */
   do
    call AddError 'Cannot change directory to: "'||AttDir||'"'  /* report */
    return ''  /* and quit empty-handed */
   end
  end
 end
end

if ((GotAttach) | (\ASCIIMessage)) then  /* if we've got MIME attachments, or we want quoted-printable text */
do
 NewHead = NewHead||'Mime-version: 1.0'||Global.CRLF  /* add the MIME version header line */
end

if (GotAttach) then  /* if we've got MIME attachments */
do
 MIMEBound = '=='||date('S')||'=='||time('L')||'=='  /* the MIME boundary to use (any unique string will do) */
 if (Attach.0 > 1) then  /* if we have more than one attachment */
 do
  Plural = 's'  /* we need a plural "s" */
 end
 else  /*if there is only one attachment */
 do
  Plural = ''  /* we don't need the "s" */
 end
 NewHead = NewHead||'Content-type: multipart/mixed; boundary="'||MIMEBound||'"'||Global.CRLF  /* add the MIME boundary header line */
 NewHead = NewHead||Global.CRLF  /* add an empty line */
 NewHead = NewHead||'This is a multi-part message in MIME format.'||Global.CRLF  /* add a plain text message line */
 NewHead = NewHead||'In addition to the message below it contains '||,  /* add an info line */
           Attach.0||' b64-encoded attachment'||Plural||'.'||Global.CRLF  /* rest of the info line */
 NewHead = NewHead||Global.CRLF||'--'||MIMEBound||Global.CRLF  /* add an empty line followed by a MIME boundary */
end

if (ASCIIMessage) then  /* if we want ASCII text */
do
 if (GotAttach) then  /* if this is a MIME-enhanced message */
 do
  NewHead = NewHead||'Content-Type: text/plain; charset="us-ascii"'||Global.CRLF  /* the first type line */
  NewHead = NewHead||'Content-Transfer-Encoding: 7bit'||Global.CRLF  /* another one */
 end
end
else  /* if we want quoted-printable text */
do
 NewHead = NewHead||'Content-Type: text/plain; charset="iso-8859-1"'||Global.CRLF  /* the first type line */
 NewHead = NewHead||'Content-Transfer-Encoding: quoted-printable'||Global.CRLF  /* another one */
end

NewHead = NewHead||Global.CRLF  /* add a separating empty line to the header */

NewBody = ''  /* start with nothing */

do while (length(MessBody) > 0)  /* go through the contents of the body text */
 parse var MessBody NextLine (Global.CRLF) MessBody  /* and each time, take the next line from the body text */
 if (ASCIIMessage) then  /* if we want ASCII text */
 do
  NextLine = translate(NextLine,' ',d2c(9))  /* convert TABs to spaces */
  do while (length(NextLine) > Global.ASCIILineLength)  /* as long as the line is longer than the limit */
   CutPoint = Global.ASCIILineLength + 1  /* start the cut at ASCIILineLength plus one */
   parse var NextLine NextBit =(CutPoint) NextLine  /* get the next bit of the maximum length */
   if ((substr(NextBit,length(NextBit),1) >< ' ') & (substr(NextLine,1,1) >< ' ')) then  /* if we did not cut the line just before or after a space, i.e. we cut through a word */
   do
    LastSpace = lastpos(' ',NextBit)  /* look for the last space in this bit */
    if (LastSpace >< 0) then  /* if we found a space */
    do
     parse var NextBit NextBit =(LastSpace) MoreBit  /* cut off the bit after the space and save it for the next line */
     MoreBit = strip(MoreBit,'L',' ')  /* get rid the leading space */
     NextLine = MoreBit||NextLine  /* stick this bit back on to the next line */
    end
   end
   else  /* if we did not cut through a word */
   do
    NextLine = strip(NextLine,'L',' ')  /* get rid of leading spaces */
   end
   NextBit = strip(NextBit,'T',' ')  /* get rid of trailing spaces */
   NewBody = NewBody||NextBit||Global.CRLF  /* add the bit we have to the new body text */
  end
  NewBody = NewBody||NextLine||Global.CRLF  /* add the remaining line to the body text */
 end
 else  /* if we don't want ASCII text, we get quoted-printable */
 do
  if (NextLine = '.') then  /* if we have a lone dot, let's prevent SMTP problems */
  do
   OutLine = '=2E'  /* turn it into a harmless =hex string */
  end
  else  /* if we have something else */
  do
   OutLine = ''  /* start with an empty line */
   do while (length(NextLine) > 0)  /* as long as we have characters left in the line */
    parse var NextLine NextChar 2 NextLine  /* get the next character from the line */
    if ((c2d(NextChar) > 126) | (NextChar = '=')) then  /* if it's not in the OK range, or if it is an equal sign */
    do
     NextChar = '='||c2x(NextChar)  /* substitute an equals sign followed by the character's hex code */
    end
    if ((length(OutLine||NextChar) > 76) | ((length(OutLine||NextChar) = 76) & (length(NextLine) > 0))) then  /* if we end up with more than 76 characters (the maximum according to the RFC spec) */
    do
     NewBody = NewBody||OutLine||'='||Global.CRLF  /* add the line to the body text, end the line with = and start a new line */
     OutLine = ''  /* we now have an empty line */
    end
    OutLine = OutLine||NextChar  /* add the new character to the new line */
   end
  end
  NewBody = NewBody||OutLine||Global.CRLF  /* add to the body, with a CRLF */
 end
end

if (\OpenFile(SendFile,'WRITE')) then  /* if we cannot open the temporary file for writing */
do
 return ''  /* no result */
end

call charout SendFile,NewHead  /* write the new header to the sendable file */
call charout SendFile,NewBody  /* write the new message body to the sendable file */

Success = 1  /* assume success */

if (GotAttach) then  /* if we've got one or more MIME attachments */
do
 Encoder = Global.ProgDir||'MMENCODE.EXE'  /* the encoder file spec */
 if (stream(Encoder,'C','QUERY EXISTS') = '') then  /* if we can't find the encoder */
 do
  call AddError 'Cannot find encoder: "'||Encoder||'"'  /* report */
  Success = 0  /* no success */
 end
 else  /* if we find the decoder */
 do Index = 1 to Attach.0  /* take each of the attachment files */
  AttachName = filespec('N',Attach.Index)  /* get the name of the attachment file */
  DotPos = lastpos('.',AttachName)  /* look for the last dos position in the file name */
  if (DotPos > 0) then  /* if we have a last dot */
  do
   AttachExt = substr(AttachName,DotPos+1)  /* get the extension */
  end
  MIMEType = GetFileEntry(MIMETypeFile,AttachExt)  /* try to match the extension with a MIME type */
  if (MIMEType >< '') then  /* if we can match it */
  do
   parse var MIMEType MIMEType '|' .  /* use the first one if there are several (which should not be the case) */
  end
  else  /* if we cannot match it (or if there is no MIME types list) */
  do
   MIMEType = 'application/octet-stream'  /* use this */
  end
  MIMEStart = Global.CRLF||Global.CRLF||'--'||MIMEBound  /* start with an empty line followed by a MIME boundary */
  MIMEStart = MIMEStart||Global.CRLF||'Content-Type: '||MIMEType||'; name="'||AttachName||'"'  /* add a content line */
  MIMEStart = MIMEStart||Global.CRLF||'Content-Transfer-Encoding: base64'  /* add another one */
  MIMEStart = MIMEStart||Global.CRLF||'Content-Disposition: attachment; filename="'||AttachName||'"'  /* and another one */
  MIMEStart = MIMEStart||Global.CRLF||Global.CRLF  /* add an empty line */
  call charout SendFile,MIMEStart  /* write it to the temporary file */
  call stream SendFile,'C','CLOSE'  /* close the new message file to allow MMENCODE.EXE access */
  Encoder||' < "'||Attach.Index||'" >> "'||SendFile||'"'  /* convert SourceFile to B64 code and add the result to SendFile [EXTERNAL] */
  if (RC >< 0) then  /* if we get an error */
  do
   Success = 0  /* we have no success overall */
  end
  call OpenFile SendFile,'WRITE'  /* reopen the new message file for more writing */
 end
 call charout SendFile,Global.CRLF||Global.CRLF||'--'||MIMEBound||'--'  /* write an empty line followed by a MIME end boundary */
end

call stream SendFile,'C','CLOSE'  /* close the new message file */

if (\Success) then  /* if something went wrong */
do
 if (sysfiledelete(SendFile) > 0) then  /* if we cannot get rid of the temp file */
 do
  call AddError 'Cannot delete "'||SendFile||'"'  /* report */
 end
 SendFile = ''  /* return nothing */
end

return SendFile  /* end of MakeSendFile */

/**********************************************************************/
MakeTitle: procedure expose Global.  /* returns new title for mail file */
/**********************************************************************/

parse arg MessFile  /* get the parameter */

TitleWords = Global.ObjectTitle  /* copy the contents of the global object title words string */

OutGoing = (MessageSettings(MessFile,'-*******','MATCH'))  /* set a message type flag for outgoing */

if (OutGoing) then  /* if the message is outgoing */
do
 EmptyStart = Global.Warning||' '  /* start of an empty line */
 EmptyEnd = ''  /* end of an empty line */
end
else  /* if the message is incoming */
do
 EmptyStart = '('  /* start of an empty line */
 EmptyEnd = ')'  /* end of an empty line */
end

TitleCount = 0  /* start at 0 */
Title. = ''  /* start with nothing */

do while (TitleWords >< '')  /* as long as we have object title words left */
 TitleCount = TitleCount + 1  /* up the title lines counter */
 parse upper var TitleWords NextWord TitleWords  /* get the next word in upper case */
 select  /* do one of the following */
  when (NextWord = 'DATE') then  /* if the next word is 'DATE' */
  do
   Title.TitleCount = GetTitleDate(MessFile)  /* get the date */
  end
  when (NextWord = 'SENDER') then  /* if the next word is 'SENDER' */
  do
   Title.TitleCount = GetTitleSender(EmptyStart,EmptyEnd,Outgoing)  /* get the sender */
  end
  when (NextWord = 'RECIPIENTS') then  /* if the next word is 'RECIPIENTS' */
  do
   Title.TitleCount = GetTitleRecipients(EmptyStart,EmptyEnd,Outgoing)  /* get the recipients */
  end
  when (NextWord = 'SUBJECT') then  /* if the next word is 'SUBJECT' */
  do
   Title.TitleCount = GetTitleSubject(EmptyStart,EmptyEnd)  /* get the subject */
  end
  otherwise  /* if it is not one of the above */
  do
   call AddError 'Illegal keyword in ObjectTitle setting: '||NextWord  /* report a non-fatal error */
   TitleCount = TitleCount - 1  /* reduce the counter back to its previous value */
  end
 end
end

if (pos(Global.CRLF||Global.Warning,Global.CRLF||Global.MessHead) > 0) then  /* if the header has a line that starts with a marker */
do
 TitleCount = TitleCount + 1  /* up the title lines counter */
 Title.TitleCount = EmptyStart||'illegal keyword'||EmptyEnd  /* add a warning */
end

if (Global.MessBody = '') then  /* if we have no message body */
do
 TitleCount = TitleCount + 1  /* up the title lines counter */
 Title.TitleCount = EmptyStart||'no message body'||EmptyEnd  /* add a warning */
end

do Index = 2 to TitleCount  /* for each line in the title, except the first one */
 Title.Index = d2c(10)||Title.Index  /* add a leading new line */
end

NewTitle. = ''  /* start with nothing */
do Index = 1 to TitleCount  /* for each line in the title */
 NewTitle.Index = Title.Index  /* copy the contents */
end

FullTitle = ''  /* start with nothing */
MaxLineLength = 100  /* the maximum no. of characters in a file name line */
do Index = 1 to TitleCount  /* for each line in the title */
 if (length(NewTitle.Index) > MaxLineLength) then  /* if this part exceeds the maximum line length */
 do
  NewTitle.Index = substr(NewTitle.Index,1,MaxLineLength)  /* reduce this one to the maximum line length */
 end
 FullTitle = FullTitle||NewTitle.Index  /* add it to the full title */
end

MaxTitleLength = 200 - length(filespec('D',MessFile)||filespec('P',MessFile))  /* the maximum no. of characters in the title */
if (length(FullTitle) > MaxTitleLength) then  /* if the resulting title is too long */
do
 CutLength = MaxTitleLength  /* start by reducing all the strings to the maximum allowable total length */
 do while (length(FullTitle) > MaxTitleLength)  /* as long as the full title is too long */
  do Index = 1 to TitleCount  /* for each line in the title */
   if (length(NewTitle.Index) > CutLength) then  /* if this part exceeds the cutting length */
   do
    NewTitle.Index = substr(NewTitle.Index,1,CutLength)  /* reduce this one to the cutting length */
   end
  end
  FullTitle = ''  /* start with nothing once more */
  do Index = 1 to TitleCount  /* for each line in the title */
   FullTitle = FullTitle||NewTitle.Index  /* add it to the full title */
  end
  CutLength = CutLength - 1  /* reduce the cutting length by one */
 end
end

FullTitle = ''  /* start with nothing once more */
do Index = 1 to TitleCount  /* for each line in the title */
 NewTitle.Index = strip(NewTitle.Index,'T',' ')  /* get rid of trailing blanks */
 if (NewTitle.Index >< Title.Index) then  /* if the contents are not what we originally had */
 do
  NewTitle.Index = NewTitle.Index||'...'  /* add an ellipsis */
 end
 FullTitle = FullTitle||NewTitle.Index  /* add it to the full title */
end

FullTitle = translate(FullTitle,'()()_________..'||"''",'<>[]~^?%&*\|/;,"'||"`")  /* remove difficult characters from the title string */

FullTitle = FullTitle||d2c(255)  /* add a hard space to prevent problems with file filter associations */

return FullTitle  /* end of MakeTitle */

/**********************************************************************/
GetTitleDate: procedure expose Global.  /* gets the date for the object title */
/**********************************************************************/

parse arg MessFile  /* get the argument */

TitleDate = GetHeaderEntry('DATE:')  /* get the date */
if ((TitleDate = '') | (TitleDate = d2c(9))) then  /* if we have no date */
do
 TitleDate = stream(MessFile,'C','QUERY TIMESTAMP')  /* use the last write date & time */
 parse var TitleDate DDate '  ' TTime  /* separate the two components */
 TimeZone = value('TZ',,'OS2ENVIRONMENT')  /* look for a TZ value */
 if (TimeZone >< '') then  /* if we find one */
 do
  TimeZone = ' '||TimeZone  /* add a space */
 end
 DoW = ' ('||substr(date('W'),1,3)||')'  /* the first three characters of the day of the week in brackets */
 TitleDate = DDate||' '||TTime||TimeZone||DoW  /* stick all together with just one space between the date and time */
end
else  /* if we have a date */
do
 TitleDate = SortedDate(TitleDate)  /* rework the date string */
end

return TitleDate  /* end of GetTitleDate */

/**********************************************************************/
GetTitleSender: procedure expose Global.  /* gets the sender for the object title */
/**********************************************************************/

parse arg EmptyStart,EmptyEnd,OutGoing  /* get the arguments */

TitleSender = GetHeaderEntry('FROM:')  /* get the sender */

if (TitleSender = '') then  /* if we have nothing */
do
 if (OutGoing) then  /* if it is an outgoing message */
 do
  TitleSender = MakeFromLine()  /* try to get a default "From:" line from the configuration file */
 end
 else  /* if it is an incoming message */
 do
  TitleSender = GetHeaderEntry('APPARENTLY-FROM:')  /* get the sender this way */
  if (TitleSender = '') then  /* if we have no sender */
  do
   TitleSender = GetHeaderEntry('SENDER:')  /* get the sender this way */
   if (TitleSender = '') then  /* if we have no sender */
   do
    TitleSender = GetHeaderEntry('FROM')  /* get the sender this way */
   end
  end
 end
end

if ((TitleSender = '') | (TitleSender = d2c(9))) then  /* if we still have no sender */
do
 TitleSender = EmptyStart||'no sender'||EmptyEnd  /* use this */
end
else  /* if we have something */
do
 TitleSender = FormatAddresses(TitleSender,0,OutGoing)  /* get a reformatted address string, no indent, check if outgoing */

 if (pos(Global.CRLF,TitleSender) > 0) then  /* if it contains more than one address */
 do
  TitleSender = Global.Warning||' multiple senders'  /* insert a warning */
 end
 else  /* if there is only one address */
 do
  if (pos(Global.Warning,TitleSender) > 0) then  /* if it contains a marker */
  do
   TitleSender = Global.Warning||' illegal sender'  /* insert a warning */
  end
  else  /* if there is no marker */
  do
   TitleSender = GetNames(TitleSender)  /* extract a list of names */
   TitleSender = DecodeHeaderLine(TitleSender)  /* decode any MIME stuff */
  end
 end
end

return TitleSender  /* end of GetTitleSender */

/**********************************************************************/
GetTitleRecipients: procedure expose Global.  /* gets the recipients for the object title */
/**********************************************************************/

parse arg EmptyStart,EmptyEnd,OutGoing  /* get the arguments */

TitleRecipients = GetHeaderEntry('TO:')  /* get the recipient */

if (TitleRecipients = '') then  /* if we have no recipient */
do
 TitleRecipients = GetHeaderEntry('APPARENTLY-TO:')  /* get the sender this way */
end

if ((TitleRecipients = '') | (TitleRecipients = d2c(9))) then  /* if we still have no recipient */
do
 TitleRecipients = EmptyStart||'no recipient'||EmptyEnd  /* use this */
end
else  /* if we have something */
do
 TitleRecipients = FormatAddresses(TitleRecipients,0,OutGoing)  /* get a reformatted address string, no indent, check if outgoing */
 if (pos(Global.Warning,TitleRecipients) > 0) then  /* if it contains a marker */
 do
  TitleRecipients = Global.Warning||' illegal recipient'  /* insert a warning */
 end
 else  /* if there is no marker */
 do
  TitleRecipients = GetNames(TitleRecipients)  /* extract a list of names */
 end
end

TitleCc = GetHeaderEntry('CC:')  /* look for Cc addresses */
if (TitleCc >< '') then  /* if CC addresses were found */
do
 TitleCc = FormatAddresses(TitleCc,0,OutGoing)  /* get a reformatted address string, no indent, check if outgoing */
 if (pos(Global.Warning,TitleCc) > 0) then  /* if it contains a marker */
 do
  TitleRecipients = Global.Warning||' illegal recipient'  /* insert a warning */
 end
 else  /* if there is no marker */
 do
  TitleCc = GetNames(TitleCc)  /* extract a list of names */
  TitleRecipients = TitleRecipients||' CC: '||TitleCc  /* add them to the list of recipients */
 end
end

TitleBcc = GetHeaderEntry('BCC:')  /* look for Bcc addresses */
if (TitleBcc >< '') then  /* if BCC addresses were found */
do
 TitleBcc = FormatAddresses(TitleBcc,0,OutGoing)  /* get a reformatted address string, no indent, check if outgoing */
 if (pos(Global.Warning,TitleBcc) > 0) then  /* if it contains a marker */
 do
  TitleRecipients = Global.Warning||' illegal recipient'  /* insert a warning */
 end
 else  /* if there is no marker */
 do
  TitleBcc = GetNames(TitleBcc)  /* extract a list of names */
  TitleRecipients = TitleRecipients||' BCC: '||TitleBcc  /* add them to the list of recipients */
 end
end

if (pos(Global.Warning,TitleRecipients) = 0) then  /* if the title contains no marker */
do
 TitleRecipients = DecodeHeaderLine(TitleRecipients)  /* decode MIME stuff */
end

return TitleRecipients  /* end of GetTitleRecipients */

/**********************************************************************/
GetTitleSubject: procedure expose Global.  /* gets the subject for the object title */
/**********************************************************************/

parse arg EmptyStart,EmptyEnd  /* get the arguments */

TitleSubject = GetHeaderEntry('SUBJECT:')  /* get the subject */

if ((TitleSubject = '') | (TitleSubject = d2c(9))) then  /* if we have no subject */
do
 TitleSubject = EmptyStart||'no subject'||EmptyEnd  /* use this */
end
else  /* if we have something */
do
 TitleSubject = DecodeHeaderLine(TitleSubject)  /* decode any MIME stuff */
end

return TitleSubject  /* end of GetTitleSubject */

/**********************************************************************/
GetNames: procedure expose Global.  /* extract a list of names from a list of addresses */
/**********************************************************************/

parse arg Addresses  /* get the argument */

Names = ''  /* start with nothing */
Separator = ','||Global.CRLF  /* the separator between full addresses */

do while (Addresses >< '')  /* as long as we have bits left */
 if (Names >< '') then  /* if we already have something */
 do
  Names = Names||'. '  /* add a separator */
 end
 parse var Addresses Address (Separator) Addresses  /* get the next address */
 if (words(Address) > 1) then  /* if the address contains more than one word */
 do
  NamePart = strip(subword(Address,1,words(Address)-1),'B','"')  /* all but the last must be the name, and get rid of any quotation marks */
  Address = subword(Address,words(Address))  /* the last is the address */
 end
 else  /* if the address contains just a single word */
 do
  NamePart = ''  /* we have no name part */
 end

 Global.TitleAddress = translate(substr(Global.TitleAddress,1,1))  /* use just the first character, in upper case */

 select  /* do one of the following */
  when (Global.TitleAddress = 'F') then  /* if we want it all */
  do
   if (NamePart >< '') then  /* if we have a name part */
   do
    NewName = NamePart||' '||Address  /* use this */
   end
   else  /* if we do not have a name part */
   do
    NewName = Address  /* use this */
   end
  end
  when (Global.TitleAddress = 'N') then  /* if we want just the name */
  do
   if (NamePart >< '') then  /* if we have a name part */
   do
    NewName = NamePart  /* use it */
   end
   else  /* if we do not have a name part */
   do
    NewName = Address  /* use the address */
   end
  end
  otherwise  /* if we want just the address */
  do
   Address = strip(Address,'L','<')  /* get rid of the leading angle bracket */
   Address = strip(Address,'T','>')  /* get rid of the trailing angle bracket */
   NewName = Address  /* use the address */
  end
 end

 Names = Names||NewName  /* add the result to the names */
end

return Names  /* end of GetNames */

/**********************************************************************/
MessageContents: procedure expose Global.  /* gets data from mail file */
/**********************************************************************/

Global.MessHead = ''  /* reset the message header content var */
Global.MessBody = ''  /* reset the message body content var */

Success = 0  /* no success yet */

parse arg MessFile  /* get the argument */

if (OpenFile(MessFile,'READ')) then  /* if we can open the message file */
do
 Global.MessBody = charin(MessFile,1,chars(MessFile))  /* copy the contents */
 call stream MessFile,'C','CLOSE'  /* and close the file */
 if (Global.MessBody >< '') then  /* if we have message content */
 do
  if (substr(Global.MessBody,1,2) = Global.CRLF) then  /* if it starts with a CRLF */
  do
   Header = ''  /* we have no header */
   Global.MessHead = ''  /* we have no header content */
  end
  else  /* if it starts with text */
  do
   parse var Global.MessBody Header (Global.EmptyLine) Global.MessBody  /* split it into a header and a body */
  end
  do while(substr(Global.MessBody,1,2) = Global.CRLF)  /* as long as the body starts with a CRLF */
   Global.MessBody = substr(Global.MessBody,3)  /* get rid of the CRLF */
  end
  if (Header >< '') then  /* if we have a header */
  do
   do while(substr(Header,length(Header)-1,2) = Global.CRLF)  /* as long as the header ends with a CRLF */
    Header = substr(Header,1,length(Header)-2)  /* get rid of the CRLF */
   end
   Header = translate(Header,' ',D2C(9))  /* convert any TABs to SPACEs */
   FoldPoint = Global.CRLF||' '  /* define a folding point */
   do until (Header = '')  /* go on util we have no header left */
    parse var Header Line (FoldPoint) Header  /* get a line */
    Global.MessHead = Global.MessHead||Line  /* add the line to the new header */
    if (Header >< '') then  /* if we have any header content left */
    do
     Header = strip(Header,'L',' ')  /* remove excess leading spaces */
     Global.MessHead = Global.MessHead||' '  /* add a single space to the new header */
    end
   end
  end
 end
 Success = 1  /* all should be well */
end

return Success  /* end of MessageContents */

/**********************************************************************/
MessageSettings: procedure expose Global.  /* handles EA settings */
/**********************************************************************/

/**********************************************************************/
/* Flags:                                                             */
/*   Position 0 :                                                     */
/*     UNSET = outgoing message                                       */
/*     SET   = incoming message                                       */
/*   Position 1 :                                                     */
/*     UNSET = not O.K. for dispatch                                  */
/*     SET   = O.K. for dispatch                                      */
/*   Position 2 :                                                     */
/*     UNSET = not processed                                          */
/*     SET   = processed                                              */
/*   Position 3 :                                                     */
/*     UNSET = non-ASCII (i.e. quoted-printable text)                 */
/*     SET   = ASCII (i.e. 7-bit, word-wrapped, no TABs)              */
/**********************************************************************/
/* Switch options:                                                    */
/*   CHANGE  : Changes the file settings to the settings provided,    */
/*             with "+" indicating "set", "-" indicating "reset", and */
/*             "*" indicating "leave unchanged".                      */
/*   MATCH   : Looks for a match between the file settings and the    */
/*             settings provided. A "*" character serves as a "don't  */
/*             care" placeholder.                                     */
/**********************************************************************/

parse upper arg MessFile,Setting,Action  /* get the arguments */

call sysgetea MessFile,'.VERSION','Version'  /* look for version */

if (Version >< '') then  /* if we found a version */
do
 parse var Version 5 CurSett  /* get the bits we want */
end
else  /* if no version was found */
do
 CurSett = '-'  /* start with '-' */
end

CurSett = left(CurSett,8,'-')  /* pad to 8 characters with "-" (i.e. "not set") */

if (Setting = '') then  /* if we received no action argument */
do
 return CurSett  /* return the current value */
end

Setting = left(Setting,8,'*')  /* pad to 8 characters with asterisks */

if (verify('Setting','*+-','M') = 0) then  /* if Setting contains only "*", "+', or "-" */
do

 do Index = 1 to 8  /* take each of the characters */
  if (substr(Setting,Index,1) = '*') then  /* if the new setting is '*' */
  do
   NewSett.Index = substr(CurSett,Index,1)  /* this position of NewSett. copies the current setting */
  end
  else  /* if the new setting is not "*" (but "+" or "-") */
  do
   NewSett.Index = substr(Setting,Index,1)  /* this position of NewSett. copies the new setting */
  end
 end
 ChgSett = ''  /* start with nothing */
 do Index = 1 to 8  /* take each of the members of NewSett. */
  ChgSett = ChgSett||NewSett.Index  /* and add it to ChgSett */
 end

 select  /* selects the action to take */

  when (Action = 'CHANGE') then  /* if we're asked to change the settings */
  do
   Success = 0  /* no success yet */
   if (NewSett.1 = '-') then  /* if the first position, i.e. the "Received" bit is not set */
   do
    TypeText = 'Mail Message Out'  /* make it an outgoing mail message file type */
   end
   else  /* if the "Received" bit is set */
   do
    TypeText = 'Mail Message In'  /* make it a received mail message file type */
   end
   EAT_MVMT = reverse('FFDF'x)  /* prepare extended attributes */
   CodePage = '0000'x  /* set code page */
   Count = reverse('0001'x)  /* set Count */
   EAT_ASCII = reverse('FFFD'x)  /* set EAT_ASCII */
   Len = reverse(right(d2c(Length(TypeText)),2,'00'x))  /* set Len */
   Type = EAT_MVMT||CodePage||Count||EAT_ASCII||Len||TypeText  /* set Type */
   if (sysputea(MessFile,'.TYPE',Type) = 0) then  /* if we attached them to the file */
   do
    Len = reverse(right(d2c(Length(ChgSett)),2,'00'x))  /* set Len */
    Version = EAT_ASCII||Len||ChgSett  /* set version */
    if (sysputea(MessFile,'.VERSION',Version) = 0) then  /* if we attached them to the file */
    do
     ChgSett = translate(ChgSett,'10','+-','0')  /* do it and pad with zeroes */
     IconFile = Global.PaperDir||'\'||substr(ChgSett,1,4)||'0000.ico'  /* get full icon file name */
     Success = (syssetobjectdata(MessFile,'TEMPLATE=NO;ICONFILE='||IconFile))  /* if we can set the icon, we're O.K. (and set template to off) */
    end
   end
   ChgSett = Success  /* return a success value */
  end

  when (Action = 'MATCH') then  /* if the action is to check for a match */
  do
   ChgSett = (ChgSett = CurSett)  /* O.K.  if they match */
  end

  otherwise  /* if we have an unexpected action parameter */
  do
   ChgSett = 0  /* return an error status */
  end

 end

end

return ChgSett  /* end of MessageSettings */

/**********************************************************************/
MoveMessage: procedure expose Global.  /* moves mail message files to subfolders */
/**********************************************************************/

parse arg MessFile  /* get the parameter */

MoveDir = ''  /* start with nothing */

Incoming = (MessageSettings(MessFile,'+*******','MATCH'))  /* is it an incoming message? */

if (InComing) then  /* if it is incoming */
do
 CompPath = Global.InDir||'\'  /* compare with the "In" folder */
 MoveFile = Global.ConfDir||'\mailread.txt'  /* the file with the "Read" folders */
end
else  /* if it is outgoing */
do
 CompPath = Global.OutDir||'\'  /* compare with the "Out" folder */
 MoveFile = Global.ConfDir||'\mailsent.txt'  /* the file with the "Sent" folders */
end

FilePath = filespec('D',MessFile)||filespec('P',MessFile)  /* the mail file path name */

if ((translate(FilePath) = translate(CompPath)) & (stream(MoveFile,'C','QUERY EXISTS') >< '')) then  /* if they are the same, i.e. the file is in the "Comp" folder, and we have a "Move" file */
do

 if (Incoming) then  /* if the message is incoming */
 do
  Address = GetHeaderEntry('FROM:')  /* get the sender's address */
  Address = FormatAddresses(Address,0,0)  /* reformat the address, no indent, no check */
  Address = word(Address,words(Address))  /* use the last word */
  Address = strip(Address,'L','<')  /* remove the leading angle bracket */
  Address = strip(Address,'T','>')  /* remove the trailing angle bracket */
  MoveDir = GetMoveDir(MoveFile,Address)  /* try to get matching a move directory */
 end

 if (MoveDir = '') then  /* if we did not find a matching move directory or if the message is outgoing */
 do
  Address = GetHeaderEntry('SENDER:')  /* get the sender's address */
  Address = FormatAddresses(Address,0,0)  /* reformat the address, no indent, no check */
  Address = word(Address,words(Address))  /* use the last word */
  Address = strip(Address,'L','<')  /* remove the leading angle bracket */
  Address = strip(Address,'T','>')  /* remove the trailing angle bracket */
  MoveDir = GetMoveDir(MoveFile,Address)  /* try to get matching a move directory */
 end

 if (MoveDir = '') then  /* if we did not find a matching move directory */
 do
  AddressList = GetHeaderEntry('TO:')  /* use the list of recipients' addresses */
  AddressList = FormatAddresses(AddressList,0,0)  /* get a formatted list of addresses, no indent, no check */
  do while ((AddressList >< '') & (MoveDir = ''))  /* as long as we have addresses left and still no move directory */
   Separator = ','||Global.CRLF  /* the separator between addresses */
   parse var AddressList Address (Separator) AddressList  /* get the next address from the list */
   Address = word(Address,words(Address))  /* use the last word */
   Address = strip(Address,'L','<')  /* remove the leading angle bracket */
   Address = strip(Address,'T','>')  /* remove the trailing angle bracket */
   MoveDir = GetMoveDir(MoveFile,Address)  /* try to find a matching move directory */
  end
 end

 if (MoveDir = '') then  /* if we did not find a matching move directory */
 do
  Subject = GetHeaderEntry('SUBJECT:')  /* use the subject line */
  do while ((Subject >< '') & (MoveDir = ''))  /* as long as we have bits of subject left and still no move directory */
   parse var Subject NextWord Subject  /* get the next word from the subject line */
   MoveDir = GetMoveDir(MoveFile,NextWord)  /* try to find matching move directories */
  end
 end

 if (MoveDir = '') then  /* if we did not find a matching move directory */
 do
  Address = '*DEFAULT*'  /* use the default entry */
  MoveDir = GetMoveDir(MoveFile,Address)  /* try to find a matching move directory */
 end

 if (MoveDir >< '') then  /* if we found a directory to move the message file to */
 do
  MoveDir = MoveMessFile(MessFile,MoveDir,Incoming)  /* see if can move the message file to the directory */
 end

end

return MoveDir  /* end of MoveMessage */

/**********************************************************************/
GetMoveDir: procedure expose Global.  /* check for a move directory in the move file */
/**********************************************************************/

MoveDir = ''  /* start with nothing */

parse arg MoveFile,SearchText  /* get the arguments */

MoveDir = GetFileEntry(MoveFile,SearchText)  /* look in the "Move" file for a folder to go with the address */
if ((MoveDir = '') & (pos('@',SearchText) > 0)) then  /* if we find nothing and if the search text contains an @ */
do
 parse var SearchText . '@' Domain  /* split the text into the bits on either side of the @ */
 MoveDir = GetFileEntry(MoveFile,Domain)  /* look in the "Move" file for a folder to go with the domain name */
end

return MoveDir  /* end of GetMoveDir */

/**********************************************************************/
MoveMessFile: procedure expose Global.  /* move a message file to a different directory */
/**********************************************************************/

parse arg MessFile,MoveDir,InComing  /* get the arguments */

if (translate(MoveDir) = '*DELETE*') then  /* if it is the special delete string */
do
 MoveDir = Global.TempDir  /* use the temp dir to move the file out of sight to be deleted the next time the mail folders are opened */
end
else  /*if it is not the special delete string */
do
 if (substr(MoveDir,1,1) = '*') then  /* if the dir name starts with this, we need to add the In or Out Archive dir */
 do
  MoveDir = strip(substr(MoveDir,2),'L','\')  /* use the rest and strip off any leading backslash */
  if (Incoming) then  /* if the message is incoming */
  do
   MoveDir = Global.InArchDir||'\'||MoveDir  /* use this destination */
  end
  else  /* if it is outgoing */
  do
   MoveDir = Global.OutArchDir||'\'||MoveDir  /* use this destination */
  end
 end
end

OrgTitle = MakeTitle(MessFile)  /* get the title of the message */

MessID = '<'||date('B')||time('L')||'>'  /* create a unique object ID */
call syssetobjectdata MessFile,'OBJECTID='||MessID||';'  /* give the message file object the unique ID */

TempName = systempfilename(MoveDir||'\RXML????.MOVE')  /* get a unique temporary file name in the move dir (so we can move the file even if an identical title exists in the target dir) */
TempTitle = filespec('N',TempName)  /* get the name bit */
call syssetobjectdata MessID,'TITLE='||TempTitle||';'  /* change the message file object title */

if (sysmoveobject(MessID,MoveDir)) then  /* if we can move the message file to the directory */
do
 call syssetobjectdata MessID,'TITLE='||OrgTitle||';'  /* change the message file object title back to what it was before the move */
end
else  /* if we cannot move the file */
do
 call AddError 'Cannot move message to "'||MoveDir||'"'  /* report */
 MoveDir = ''  /* return nothing */
end

return MoveDir  /* end of MoveMessFile */

/**********************************************************************/
OpenFolders: procedure expose Global.  /* opens the RexxMail message folders */
/**********************************************************************/

if (\sysopenobject(Global.MainDir,0,1)) then  /* if we cannot open the main dir */
do
 call AddError 'Cannot open RexxMail folder "'||Global.MainDir||'"'  /* report */
 return 0  /* and quit */
end

call sysopenobject Global.AddrDir,0,1  /* open the "Addresses" dir */
call sysopenobject Global.InDir,0,1  /* open the "In" dir */
call sysopenobject Global.OutDir,0,1  /* open the "Out" dir */
call sysopenobject Global.InArchDir,0,1  /* open the "In Archive" dir */
call sysopenobject Global.OutArchDir,0,1  /* open the "Out Archive" dir */
call MailWarning 'DELETE'  /* destroy the program object, if any */

if (Global.CleanupTemp) then  /* if we are to clean up the Temp dir */
do
 call sysfiletree Global.OutDir||'\*.*','Files.','FO'  /* look for files in the outgoing mail message directory */
 AllAttDirs = ''  /* start with nothing */
 do Index = 1 to Files.0  /* take each of the objects found */
  AllAttDirs = AllAttDirs||' '||SetAttachments(Files.Index,'GETDIR')  /* get the file's attachment folder name, if any, and add it to the list */
 end

 call sysfiletree Global.TempDir||'\RXML????','Dirs.','DO'  /* look for RexxMail attachment folders in the temporary storage directory */
 do Index = 1 to Dirs.0  /* take each of the objects found */
 if (wordpos(Dirs.Index,AllAttDirs) = 0) then  /* if its name is not in the current attachment folders list */
  do
   call sysfiletree Dirs.Index||'\*.*','Files.','FOS'  /* look for files in the folder and its subfolders */
   do SubIndex = 1 to Files.0  /* take each of the objects found */
   call sysfiledelete Files.SubIndex  /* try to delete it */
   end
   call sysfiletree Dirs.Index||'\*.*','SubDirs.','DOS'  /* look for directories in the folder and its subfolders */
   do SubIndex = SubDirs.0 to 1 by -1  /* take each of the directories found, in reverse order */
   call sysrmdir SubDirs.SubIndex  /* try to delete it */
   end
   call sysrmdir Dirs.Index  /* try to delete the base folder */
  end
 end

 call sysfiletree Global.TempDir||'\*.*','Files.','FO'  /* look for files in the temporary storage directory */
 do Index = 1 to Files.0  /* take each of the objects found */
  call sysfiledelete Files.Index  /* try to delete it */
 end

end
return 1  /* end of OpenFolders */

/**********************************************************************/
OpenInfo: procedure expose Global.  /* opens the Info index file */
/**********************************************************************/

Success = sysopenobject(Global.InfoDir||'\index.html',0,1)  /* open the index file */

return Success  /* that's all, folks! */

/**********************************************************************/
RegisterMessage: procedure expose Global.  /* registers incoming messages */
/**********************************************************************/

parse arg MessFile  /* get the argument */

call MessageContents MessFile  /* get the file contents */

if (\MessageSettings(MessFile,'+-------','CHANGE')) then  /* if we cannot attach the unread received mail message setting */
do
 call AddError 'Cannot set mail message attributes of "'||MessFile||'"'  /* report */
 return 0  /* and quit */
end

if (\syscopyobject(MessFile,Global.InDir)) then  /* if the mail file cannot be copied to the In directory ("sysmoveobject" does not always set the EAs) */
do
 call AddError 'Cannot copy "'||MessFile||'" to "'||Global.Indir||'"'  /* report */
 return 0  /* and quit */
end

if (sysfiledelete(MessFile) > 0) then  /* if we cannot get rid of the original file */
do
 call AddError 'Cannot delete "'||MessFile||'"'  /* report */
end

MessFile = Global.InDir||'\'||filespec('N',MessFile)  /* use a new file spec */

NewName = MakeTitle(MessFile)  /* get the new name for the mail file */

call LogMessage  /* log the message without a date argument */

if (\syssetobjectdata(MessFile,'TITLE='||NewName)) then  /* if we cannot set the new file title */
do
 call AddError 'Cannot set message file title'  /* report */
end

if (Global.Signals) then  /* if a signal is wanted */
do
 call beep 888,50  /* signal */
 call beep 444,100  /* signal */
end

call MailWarning  /* create a warning icon if necessary */

return 1  /* end of RegisterMessage */

/**********************************************************************/
SetAttachments: procedure expose Global.  /* handles attachment file folders and settings */
/**********************************************************************/

/**********************************************************************/
/* List of switch options:                                            */
/* [none] : Looks for an existing attachments folder. If no reference */
/*          is found, or if the folder no longer exists, creates an   */
/*          attachments folder and returns its name; otherwise,       */
/*          returns the name of the existing folder.                  */
/* CREATE : Create a new attachments folder and set the EA.           */
/*          Returns the full name of the new folder.                  */
/* DELETE : Delete the attachments folder EA.                         */
/*          Returns 1 on success, otherwise 0                         */
/* GETDIR : The attachment folder name is retrieved from the EAs.     */
/*          Returns the folder name.                                  */
/* LIST   : An attachment list is retrieved.                          */
/*          Returns the list of attachments.                          */
/* SEND   : An attachment list is created, and the folder EA deleted. */
/*          Returns 1 on success, otherwise 0                         */
/**********************************************************************/

parse arg MessFile,Option,DateString  /* get the parameters */

if (Option = '') then  /* if no option was passed */
do
 AttDir = MailAttGetDir(MessFile)  /* get the name of the attachments directory, if any */
 if (AttDir = '') then  /* if no directory name was found */
 do
  AttDir = MailAttCreate(MessFile)  /* create it */
 end
 else  /* if a directory name was returned */
 do
  call sysfiletree AttDir,'Dirs','DO'  /* look for the actual directory */
  if (Dirs.0 = 0) then  /* if no such directory was found (i.e. it was deleted) */
  do
   AttDir = MailAttCreate(MessFile)  /* create it */
  end
 end
 Result = AttDir  /* return attachments folder name */
end
else  /* if an option was passed */
do
 Option = translate(Option)  /* make it uppercase */

 select  /* we have the following options */

  when (Option = 'CREATE') then  /* if a new attachments folder is to be created */
  do
   Result = MailAttCreate(MessFile)  /* if we can create a new attachments folder, we're O.K. */
  end

  when (Option = 'DELETE') then  /* if the attachment dir is to be deleted */
  do
   Result = 0  /* no result yet */
   if (MailAttGetDir(MessFile) >< '') then  /* if there is an attachment directory */
   do
    Result = MailAttDelete(MessFile,AttDir)  /* if we can delete the attachments directory EA, we're O.K. */
   end
  end

  when (Option = 'GETDIR') then  /* if the option is GetDir */
  do
   Result = MailAttGetDir(MessFile)  /* go get the attachment directory name, if any */
  end

  when (Option = 'LIST') then  /* if the message file was sent */
  do
   Result = MailGetList(MessFile)  /* get the attachments list */
  end

  when (Option = 'SEND') then  /* if the message file was sent */
  do
   Result = MailAttList(MessFile,MailAttGetDir(MessFile),DateString)  /* if we can set the date and the attachments list EA, we're O.K. */
  end

  otherwise  /* if it is an unknown option */
  do
   nop  /* do nothing */
  end

 end
end

return Result  /* that's all, folks! */

/**********************************************************************/
MailAttCreate: procedure expose Global.  /* create an attachment dir and EA */
/**********************************************************************/

parse arg MessFile  /* get the argument */

AttDir = systempfilename(Global.TempDir||'\RXML????')  /* get a unique directory name in the temp files directory */
ObjTitle = filespec('N',AttDir)  /* get the name bit */
Settings = 'WORKAREA=YES;ALWAYSSORT=YES;BACKGROUND=(none),,,C,204 204 0'  /* make it a work area, sort the folder contents and set the background colour */

if (syscreateobject('WPFolder',ObjTitle,Global.TempDir,Settings)) then  /* if we created the directory */
do
 Len = reverse(right(d2c(Length(AttDir)),2,'00'x))  /* set Len */
 Subject = reverse('FFFD'x)||Len||AttDir  /* prepare Subject */
 call sysputea MessFile,'.SUBJECT',Subject  /* attach it to the file */
end
else  /* if we could not create the directory */
do
 AttDir = ''  /* nothing to return */
end

return AttDir  /* end of MailAttCreate */

/**********************************************************************/
MailAttDelete: procedure expose Global.  /* delete the attachments directory and EA */
/**********************************************************************/

parse arg MessFile,AttDir  /* get the arguments */

Subject = reverse('FFFD'x)||'0000'x  /* prepare an empty Subject */
Success = (sysputea(MessFile,'.SUBJECT',Subject) = 0)  /* if we can set it (i.e. delete the Subject EA), we're O.K. */

return Success  /* end of MailAttDelete */

/**********************************************************************/
MailAttGetDir: procedure expose Global.  /* get the attachment directory name */
/**********************************************************************/

parse arg MessFile  /* get the argument */

AttDir = ''  /* start with zilch */

call sysgetea MessFile,'.SUBJECT','Subject'  /* look for Subject */
if (Subject >< '') then  /* if we found a subject */
do
 parse var Subject EAT 3 Len 5 Text  /* get the bits we want */
 if (EAT = reverse('FFFD'x)) then  /* if it is ASCII text */
 do
  AttDir = Text  /* the text must be the attachment directory */
 end
end

return AttDir  /* end of MailAttGetDir */

/**********************************************************************/
MailAttList: procedure expose Global.  /* sets a list of mail attachments */
/**********************************************************************/

parse arg MessFile,AttDir,DateString  /* get the arguments */

Subject = reverse('FFFD'x)||'0000'x  /* prepare an empty Subject */
if (\sysputea(MessFile,'.SUBJECT',Subject) = 0) then  /* if we cannot set it (i.e. delete the Subject EA) */
do
 return 0  /* return with no success */
end

Contents. = ''  /* clear the contents array */
Count = 0  /* we have no lines so far */

Count = Count + 1  /* up the counter */
Contents.Count = 'Sent:'  /* the first line of text */

Count = Count + 1  /* up the counter */
Contents.Count = ' '||DateString  /* add the date string */

if (AttDir >< '') then  /* if we have an attachments dir */
do
 call sysfiletree AttDir||'\*.*','Attach','FO'  /* look for attachment files in the attachments directory */
 if (Attach.0 > 0) then  /* if we have attachments */
 do
  if (Attach.0 > 1) then  /* if we have more than one */
  do
   Plural = 's'  /* we need a plural "s" */
  end
  else  /* if there is just one attachment */
  do
   Plural = ''  /* we do not need a plural "s" */
  end
  Count = Count + 1  /* up the counter */
  Contents.Count = 'Attachment'||Plural||':'  /* a new line of text to add */
  do Index = 1 to Attach.0  /* take each of the remaining attachment files, if any */
   Count = Count + 1  /* up the counter */
   Contents.Count = ' '||filespec('N',Attach.Index)  /* add the attachment file name with a leading space */
  end
 end
end

NewContents = ''  /* start with nothing */
do Index = 1 to Count  /* run through the text we have */
 Len = reverse(right(d2c(Length(Contents.Index)),2,'00'x))  /* set Len */
 NewContents = NewContents||reverse('FFFD'x)||Len||Contents.Index  /* add to the contents */
end

Count = reverse(right(d2c((Count)),2,'00'x))  /* set Count */
NewContents = reverse('FFDF'x)||'0000'x||Count||NewContents  /* prepare the rest of the contents */
Success = (sysputea(MessFile,'.COMMENTS',NewContents) = 0)  /* if we can attach them to the file, we're O.K. */

return Success  /* end of MailAttList */

/**********************************************************************/
MailGetList: procedure expose Global.  /* gets a list of mail attachments */
/**********************************************************************/

parse arg MessFile  /* get the argument */

AttList = ''  /* start with zilch */

call sysgetea MessFile,'.COMMENTS','Comments'  /* look for Comments */
if (Comments >< '') then  /* if we found comments */
do
 parse var Comments EAT 3 . 5 Count 7 Contents  /* get the bits we want */
 if (EAT = reverse('FFDF'x)) then  /* if it is a multi-value, multi-type entry */
 do
  Count = c2d(reverse(Count))  /* restore the count value to the right order */
  do Index = 1 to Count  /* run through them all */
   parse var Contents EAT 3 Len 5 Contents  /* get the bits we want  */
   if (EAT = reverse('FFFD'x)) then  /* if it is ASCII text */
   do
    Next = c2d(reverse(Len)) + 1  /* where the next entry starts */
    parse var Contents Text =(Next) Contents  /* get the bits we need */
    if (substr(Text,1,1) = ' ') then  /* if the text starts with a space */
    do
     if (GotSpace) then  /* if we already have an entry that started with a space */
     do
      Text = ','||Text  /* stick a comma in front of this bit */
     end
     GotSpace = 1  /* we have a space */
    end
    else  /* if the text does not start with a space */
    do
     Text = Global.CRLF||Text  /* start with a new line */
     GotSpace = 0  /* we have no space this time */
    end
    AttList = AttList||Text  /* add the text to the list */
   end
  end
 end
end

return AttList  /* end of MailGetList */

/**********************************************************************/
SetMessageType: procedure expose Global.  /* sets message type attributes */
/**********************************************************************/

parse arg MessFile,Switch  /* get the arguments */

Success = 0  /* no success yet */

SwitchString = 'SETNEW SETDISPATCHED SETRECEIVED SETVIEWED SETASCII SETNONASCII'  /* the available types */
SettingsString = '----**** --+***** +--***** +-+***** ***+**** ***-****'  /* the corresponding settings strings */
SetTitleString = 'SETNEW SETDISPATCHED SETRECEIVED SETVIEWED'  /* change the title as well for these */

Switch = translate(Switch)  /* convert to upper case */

Settings = word(SettingsString,wordpos(Switch,SwitchString))  /* get the settings that correspond with the message type */

if (MessageSettings(MessFile,Settings,'CHANGE')) then  /* if we can attach the mail message settings */
do
 if (wordpos(Switch,SetTitleString) > 0) then  /* if the switch is in the list for which the title should be set */
 do
  call MessageContents MessFile  /* get the file contents */
  NewName = MakeTitle(MessFile)  /* get a new name for the mail file */
  TitleSetting = 'TITLE='||NewName  /* the title setting to use */
 end
 else  /* if the switch is not in the title set list */
 do
  TitleSetting = ''  /* we have no title setting */
 end
 if (syssetobjectdata(MessFile,'NODELETE=NO;NODRAG=NO;NOMOVE=NO;'||TitleSetting)) then  /* if we can set the file attributes */
 do
  Success = 1  /* we're O.K. */
 end
end

return Success  /* end of SetViewed */

/**********************************************************************/
SortedDate: procedure expose Global.  /* returns sorted mail message date string */
/**********************************************************************/

parse arg DateStr  /* get the argument */

if (Global.ISODates) then  /* if we want the date sorted according to ISO specs */
do
 parse var DateStr DoW Rest  /* extract all the bits we need */
 
 if (substr(DoW,length(DoW)) = ',') then  /* if a DoW reference was included, it should be followed by a comma */
 do
  DoW = ' ('||substr(DoW,1,3)||')'  /* use the first 3 characters, between brackets, and with a leading space */
  parse var Rest Day Mon Year Time Zone  /* extract all the bits we need */
 end
 else  /* if there was no DoW reference */
 do
  Day = DoW  /* the first bit was the day of the month */
  DoW = ''  /* we have no DoW */
  parse var Rest Mon Year Time Zone  /* extract all the bits we need */
 end
 
 if (Year < 100) then  /* if Year has only 2 digits */
 do
  if (Year < 70) then  /* if it's less than 70 */
  do
   Year = '20'||Year  /* it must be the 21st century */
  end
  else  /* if it's 70 or over */
  do
   Year = '19'||Year  /* it must be the 20th century */
  end
 end
 
 Months = 'JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC'  /* the first three letters of the months in upper case */
 Month = wordpos(translate(substr(Mon,1,3)),Months)  /* convert Mon to Month figure */
 
 Month = right(Month,2,'0')  /* pad it with 0 if necessary */
 
 Day = right(Day,2,'0')  /* pad it with 0 if necessary */
 
 NewDate = Year||'-'||Month||'-'||Day||' '||Time||' '||Zone||DoW  /* this is our new date & time string */
end
else  /* if we don't want the date sorted */
do
 NewDate = DateStr  /* just use the original date string */
end

return NewDate  /* that's all, folks! */

/**********************************************************************/
UnpackMessage: procedure expose Global.  /* unpacks received mail files */
/**********************************************************************/

parse arg MessFile  /* get the mail file and attachment dir names */

AttDir = SetAttachments(MessFile,'CREATE')  /* get the name of a new attachments folder */
ReadFile = AttDir||'.READ'  /* the name for the reader file */

call MessageContents MessFile  /* extract the message contents */

HeadText = MakeHeadLines(Global.ReaderHeader)  /* get the necessary header info from the mail file */
BodyText = ''  /* start with nothing */

/**********************************************************************/
/* The following 6 lines of code are needed to work around a bug in   */
/* the "munpack.exe" decoder program.                                 */
/**********************************************************************/

MIME = (GetHeaderEntry('Mime-version:') >< '')  /* it is a MIME message */
NoCTE = (GetHeaderEntry('Content-transfer-encoding:') = '')  /* there is no "content-transfer-encoding" line */
XBit = (pos('BIT',translate(GetHeaderEntry('Content-transfer-encoding:'))) > 0)  /* it is a (MIME) ASCII message */
UUE = (pos('UUENCODE',translate(GetHeaderEntry('Encoding:'))) > 0)  /* some mailers still send UUE attachments */
PlainText = (pos('TEXT/PLAIN',translate(GetHeaderEntry('Content-type:'))) > 0)  /* it is a plain text message */

if ((MIME & \(PlainText & (XBit | NoCTE))) | (UUE)) then  /* if the message is MIME, but not ASCII plain text (i.e. only if there is something to decode), or if it contains a UUE attachment */
do

 Decoder = Global.ProgDir||'MUNPACK.EXE'  /* the decoder program spec */

 if (stream(Decoder,'C','QUERY EXISTS') = '') then  /* if the decoder does not exist */
 do
  call AddError 'Cannot find decoder: "'||Decoder||'"'  /* report */
 end
 else  /* if we can find the decoder */
 do

  Decoder||' -C '||AttDir||' -e -q -t "'||MessFile||'"'  /* unpack the mail file into the directory, adding MIME type EAs, quiet, and including text parts [EXTERNAL] */

  call sysfiletree AttDir||'\*.*','AttFiles.','FO'  /* have we got any MIME text part files? */
  if (AttFiles.0 > 0) then  /* if so */
  do Index = 1 to AttFiles.0  /* run through the lot */

   AttFileName = filespec('N',AttFiles.Index)  /* get the file name bit */

   if (sysgetea(AttFiles.Index,'MIME-TYPE','TypeInfo') = 0) then  /* if we can get the MIME type from the file's EAs */
   do
    parse var TypeInfo 5 MIMEType  /* get the bit we want */
   end

   select  /* do one of the following */

    when (translate(MIMEType) = 'TEXT/PLAIN') then  /* if it's plain text */
    do
     NamePart = filespec('N',AttFiles.Index)  /* get the name part */
     if ((translate(substr(NamePart,1,4)) = 'PART') & (datatype(substr(NamePart,5),'W'))) then  /* if it's a message part file */
     do
      if (OpenFile(AttFiles.Index,'READ')) then  /* if we can open the file */
      do
       Buffer = charin(AttFiles.Index,1,chars(AttFiles.Index))  /* copy the file contents to a buffer */
       call stream AttFiles.Index,'C','CLOSE'  /* close the part file */
       do while (substr(Buffer,1,2) = Global.CRLF)  /* as long as the text starts with a CRLF */
        Buffer = substr(Buffer,3)  /* skip the CRLF and take the rest */
       end
       if (Buffer >< '') then  /* if we have anything left */
       do
        Buffer = translate(Buffer,Global.ISOto850,xrange('80'x,'FF'x))  /* just assume it's ISO-8859-1 text and convert any high-bit characters to PC850 */
        BodyText = BodyText||Buffer||Global.CRLF  /* add the file contents to the body text */
       end
      end
      if (sysfiledelete(AttFiles.Index) > 0) then  /* if we cannot zap the part file */
      do
       call AddError 'Cannot delete "'||AttFiles.Index||'"'  /* report */
      end
     end
    end

    when (translate(MIMEType) = 'TEXT/HTML') then  /* if it's HTML */
    do
     DotPos = lastpos('.',AttFileName)  /* get the position of the last '.' in the file name, if any */
     if (DotPos > 0) then  /* if it was found */
     do
      AttExt = substr(AttFileName,(DotPos + 1))  /* get the extension of the file name */
      if (pos('HTM',translate(AttExt)) = 0) then  /* if it is not an (*)HTM(L) extension */
      do
       call syssetobjectdata AttFiles.Index,'TITLE='||filespec('N',AttFiles.Index)||'.html;'  /* add an "html" extension */
      end
     end
     else  /* if none was found */
     do
      call syssetobjectdata AttFiles.Index,'TITLE='||filespec('N',AttFiles.Index)||'.html;'  /* add an "html" extension */
     end
    end

    otherwise  /* if none of the above apply */
    do
     if ((substr(AttFileName,1,4) = 'part') & (datatype(substr(AttFileName,5),'W'))) then  /* if it is a "part*" file, with "*" a whole number */
     do
      call syssetobjectdata AttFiles.Index,'TITLE='||MIMEType  /* change the title to show what's in it */
     end
     else  /* if it's not a "part?" file */
     do
      DotPos = lastpos('.',AttFileName)  /* get the position of the last '.' in the file name, if any */
      if (DotPos > 0) then  /* if it was found */
      do
       Extension = translate(substr(AttFileName,DotPos+1))  /* get the file extension in upper case */
       if (wordpos(Extension,translate(Global.Names83)) > 0) then  /* if it is a listed extension */
       do
        Name = translate(substr(AttFileName,1,DotPos-1))  /* get the file name in upper case */
        Name = translate(Name,'                ','<>[]~?%&*\|/;,`"')  /* change dangerous characters into spaces */
        NewName = ''  /* start with nothing */
        do WIndex = 1 to words(Name)  /* run through the words in the name part */
         NewName = NewName||word(Name,WIndex)  /* and add it to the new name */
        end
        if (length(NewName) > 8) then  /* if the file name is too long */
        do
         NewName = substr(NewName,1,8)  /* get the first 8 characters of the filename */
         NewName = NewName||'.'||Extension  /* create a new 8.3 file name */
         if (stream(AttDir||'\'||NewName,'C','QUERY EXISTS') >< '') then  /* if this 8.3 file name already exists */
         do
          Counter = 0  /* start at 0 */
          do until (stream(AttDir||'\'||NewName,'C','QUERY EXISTS') = '')  /* go on until we have a file name that is new */
           Counter = Counter + 1  /* up the counter */
           NewName = substr(Name,1,8-length(Counter))||Counter||'.'||Extension  /* create a new file name */
          end
         end
         call syssetobjectdata AttFiles.Index,'TITLE='||NewName  /* change the object's title */
         HeadText = HeadText||'[Renamed attachment "'||AttFileName||'" to "'||NewName||'"]'||Global.CRLF||Global.CRLF  /* add an info line to the reader message header text */
        end
       end
      end
     end
    end

   end

  end

 end

end

if (BodyText = '') then  /* if we still have no body text */
do
 if (translate(substr(word(Global.MessBody,1),1,6)) = '<HTML>') then  /* if the message starts with this */
 do
  HTMLFile = systempfilename(AttDir||'\RXML????.HTML')  /* get a unique name to use */
  if (OpenFile(HTMLFile,'WRITE')) then  /* if we can open the file for writing */
  do
   call charout HTMLFile,Global.MessBody  /* write the body text to the file */
   call stream HTMLFile,'C','CLOSE'  /* close the file */
   BodyText = '[HTML content written to : '||HTMLFile||']'  /* the message to insert as body text */
  end
 end
 else  /* if the message does not start with the <HTML> tag */
 do
  BodyText = Global.MessBody  /* just use the original body text from the mail message */
 end
end

ReadText = HeadText||BodyText  /* the complete text to read */

if (OpenFile(ReadFile,'WRITE')) then  /* if we can open the new read file for writing */
do
 call charout ReadFile,ReadText  /* copy the buffer into the file */
 call stream ReadFile,'C','CLOSE'  /* close the file */
end

return AttDir  /* end of UnpackMessage */

/**********************************************************************/
ViewMessage: procedure expose Global.  /* views received mail messages */
/**********************************************************************/

parse arg MessFile  /* get the argument */

call MessageSettings MessFile,'**+*****','CHANGE'  /* new read mail message setting: Processed */

AttDir = UnpackMessage(MessFile)  /* unpack the mail file and get the attachments directory name */
ReadFile = AttDir||'.READ'  /* the reader file name */

call sysfiletree AttDir||'\*.*','PreFiles.','FOS'  /* see what's in the attachments dir */

if (PreFiles.0 > 0) then  /* if the folder contains files */
do

 if (Global.RunAttachIn >< '') then  /* if we have a command to run in the attachments folder */
 do

  CurDir = directory()  /* store the current directory name */

  if (directory(AttDir) = '') then  /* if we cannot change to the attachments directory */
  do
   call AddError 'Cannot change to "'||AttDir||'"'  /* report */
   return ''  /* and quit */
  end

  parse var Global.RunAttachIn FirstBit '%N' LastBit  /* look for the filename macro in the RunAttachIn command line */
  CommLine = FirstBit  /* this is the command to start with */
  if (pos('%N',Global.RunAttachIn) > 0) then  /* if the settings line contains a file spec macro */
  do
   CommLine = CommLine||date('D')||time('S')||LastBit  /* this is the complete command */
  end
  say 'RunAttachIn configuration option parsed as : '||CommLine  /* report */
  CommLine  /* run the command */

  call directory CurDir  /* change back to the original directory */

 end

 call sysfiletree AttDir||'\*.*','PreFiles.','FOS','*****','-****'  /* see what's in the attachments dir now, resetting all archive attributes */

 if (PreFiles.0 > 0) then  /* if the folder still contains files */
 do
  call sysopenobject AttDir,1,1  /* open the attachments folder on the desktop */
 end

end

parse var Global.Reader FirstBit '%N' LastBit  /* look for the filename placeholder in the reader command line */
CommLine = FirstBit  /* this is the command to start with */

if (pos('%N',Global.Reader) > 0) then  /* if the settings line contains a file spec macro */
do
 CommLine = CommLine||ReadFile||LastBit  /* this is the complete command */
end

say 'Reader configuration option parsed as : '||CommLine  /* report */
CommLine  /* run the command */

Changed = 0  /* no changes yet */

call sysfiletree AttDir||'\*.*','PostFiles.','FOS'  /* what have we got (left) in the attachments dir? */

do Index = 1 to PostFiles.0  /* run through the list of files we have now */
 Existing = 0  /* assume the file is not an existing one (i.e. it was not there before) */
 do Counter = 1 to PreFiles.0  /* run through the list of files we had before */
  if (PostFiles.Index = PreFiles.Counter) then  /* if we have a match */
  do
   call sysfiletree PostFiles.Index,'ArchFiles.','FO','+****'  /* see if the archive attribute of the file was set */
   Existing = (ArchFiles.0 = 0)  /* if it has not, the file hasn't changed */
  end
 end
 if (\Existing) then  /* if the file was not an existing one */
 do
  Changed = 1  /* we have at least 1 change */
 end
end

if (\Changed) then  /* if we have no important changes */
do
 if (\sysdestroyobject(AttDir)) then  /* if we cannot zap the attachments folder */
 do
  call AddError 'Cannot delete "'||AttDir||'"'  /* report */
 end
end

if (stream(ReadFile,'C','QUERY EXISTS') >< '') then  /* if the read file still exists */
do
 if (sysfiledelete(ReadFile) > 0) then  /* if we cannot get rid of the read file */
 do
  call AddError 'Cannot delete '||ReadFile  /* report */
 end
end

if (stream(MessFile,'C','QUERY EXISTS') >< '') then  /* if the message file still exists */
do
 call MoveMessage MessFile  /* move the file to an archive folder if needed */
end

return 1  /* end of ViewMessage */

/**********************************************************************/
ViewRawMessage: procedure expose Global.  /* shows the raw message contents */
/**********************************************************************/

parse arg MessFile  /* look for a FileName */

TempFile = systempfilename(Global.TempDir'\RXML????.RAW')  /* get a temp file name */

if (\OpenFile(TempFile,'WRITE')) then  /* if we cannot open the temp file for writing */
do
 return 0  /* and quit */
end

if (\OpenFile(MessFile,'READ')) then  /* if we cannot open the message file for writing */
do
 return 0  /* and quit */
end

call charout TempFile,charin(MessFile,1,chars(MessFile))  /* copy the whole contents of the message file into the temp file */

call stream TempFile,'C','CLOSE'  /* close the temp file */

call stream MessFile,'C','CLOSE'  /* close the message file */

if (MessageSettings(MessFile,'**-*****','MATCH')) then  /* if the message is unprocessed */
do
 call MessageSettings MessFile,'**+*****','CHANGE'  /* new edited mail message setting: Processed */
end

parse var Global.Reader FirstBit '%N' LastBit  /* look for the filename placeholder in the reader command line */

CommLine = FirstBit  /* this is the command to start with */

if (pos('%N',Global.Reader) > 0) then  /* if the settings line contains a file spec macro */
do
 CommLine = CommLine||TempFile||LastBit  /* this is the complete command */
end

say 'Reader configuration option parsed as : '||CommLine  /* report */
CommLine  /* run the command */

call sysfiledelete TempFile  /* get rid of the temporary file */

return 1  /* end of ViewRawMessage */
